<!-- ***************************************** -->
<!-- XSLT transformation for converting from   -->
<!-- U2 XML storage format 2.0 to 2.1          -->
<!-- ***************************************** -->

<xsl:transform version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<!-- Change file format version number to 2.1 -->

<xsl:template match="u2">
  <xsl:copy>
    <xsl:for-each select="@*">
    <xsl:call-template name="transform-u2-attributes"/>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template name="transform-u2-attributes">
      <xsl:choose>
        <xsl:when test="local-name()='OMVersion'">
          <xsl:attribute name="OMVersion">2.1</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
</xsl:template>

<!-- New connector model -->
<!-- remove no longer used descendants -->
<xsl:template match = "Connector/rConnection | Connector/rCarried1 | Connector/rCarried2">
</xsl:template>

<!-- remove Enabled attributes, which were moved to ConnectorEnd (and Derived in the same batch)-->
<xsl:template match = "Connector">
  <xsl:copy>
    <xsl:for-each select="@*">
	  <xsl:choose>
		<xsl:when test="local-name()='Enabled1'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:when test="local-name()='Enabled2'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:when test="local-name()='Derived'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:otherwise>
		  <xsl:copy/>
		</xsl:otherwise>
	  </xsl:choose>
    </xsl:for-each>
	<cConnectorEnd>
	  <xsl:choose>
	    <!-- Leave connector end order as it is -->
	    <xsl:when test = "count(./rEnvConnection) &lt; 1">
		  <xsl:for-each select = "./rConnection[1]">
		    <xsl:call-template name = "first-connection-transform"/>
		  </xsl:for-each>
		  <xsl:for-each select = "./rConnection[2]">
		    <xsl:call-template name = "second-connection-transform"/>
		  </xsl:for-each>
		</xsl:when>
		<!-- Put non-env connector end first -->
		<xsl:otherwise>
		  <xsl:for-each select = "./rConnection[1]">
		    <xsl:call-template name = "first-connection-transform"/>
		  </xsl:for-each>
		  <xsl:for-each select = "./rEnvConnection">
		    <xsl:call-template name = "env-connection-transform"/>
		  </xsl:for-each>
		</xsl:otherwise>
	  </xsl:choose>
	</cConnectorEnd>
  </xsl:copy>
</xsl:template>

<!-- convert old representation of connection points to new - EnvConnection -->
<xsl:template name = "env-connection-transform">
  <ConnectorEnd>
	<xsl:choose>
	  <xsl:when test = "count(../@Enabled2) &lt; 1">
		<xsl:attribute name = "Enabled">false</xsl:attribute>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:attribute name = "Enabled"><xsl:value-of select = "../@Enabled2"/></xsl:attribute>
	  </xsl:otherwise>
	</xsl:choose>
	<rPort>
      <xsl:call-template name="copy-all-attributes"/>
      <xsl:apply-templates/>
	</rPort>
    <xsl:for-each select = "../rCarried2">
      <rCarried>
	    <xsl:attribute name = "R"><xsl:value-of select = "@R"/></xsl:attribute>
	  </rCarried>
    </xsl:for-each>
  </ConnectorEnd>
</xsl:template>

<xsl:template name = "first-connection-transform">
  <ConnectorEnd>
	<xsl:choose>
	  <xsl:when test = "count(../@Enabled1) &lt; 1">
		<xsl:attribute name = "Enabled">false</xsl:attribute>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:attribute name = "Enabled"><xsl:value-of select = "../@Enabled1"/></xsl:attribute>
	  </xsl:otherwise>
	</xsl:choose>
	<rPart>
	  <xsl:call-template name = "compute-connector-part-guid">
	    <xsl:with-param name = "guid-value"><xsl:value-of select = "@R"/></xsl:with-param>
	  </xsl:call-template>
	</rPart>
    <xsl:call-template name = "compute-connector-port-guid">
      <xsl:with-param name = "guid-value"><xsl:value-of select = "@R"/></xsl:with-param>
    </xsl:call-template>
    <xsl:for-each select = "../rCarried1">
      <rCarried>
	    <xsl:attribute name = "R"><xsl:value-of select = "@R"/></xsl:attribute>
	  </rCarried>
    </xsl:for-each>
  </ConnectorEnd>
</xsl:template>

<xsl:template name="second-connection-transform">
  <ConnectorEnd>
	<xsl:choose>
	  <xsl:when test = "count(../@Enabled2) &lt; 1">
		<xsl:attribute name = "Enabled">false</xsl:attribute>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:attribute name = "Enabled"><xsl:value-of select = "../@Enabled2"/></xsl:attribute>
	  </xsl:otherwise>
	</xsl:choose>
	<rPart>
	  <xsl:call-template name = "compute-connector-part-guid">
	    <xsl:with-param name = "guid-value"><xsl:value-of select = "@R"/></xsl:with-param>
	  </xsl:call-template>
	</rPart>
    <xsl:call-template name = "compute-connector-port-guid">
      <xsl:with-param name = "guid-value"><xsl:value-of select = "@R"/></xsl:with-param>
    </xsl:call-template>
    <xsl:for-each select = "../rCarried2">
      <rCarried>
	    <xsl:attribute name = "R"><xsl:value-of select = "@R"/></xsl:attribute>
	  </rCarried>
    </xsl:for-each>
  </ConnectorEnd>
</xsl:template>

<!-- computes connector end rPart reference from rConnection -->
<xsl:template name = "compute-connector-part-guid">
  <xsl:param name = "guid-value"> none </xsl:param>
  <xsl:choose>
    <!-- check, if we have ref: reference -->
    <xsl:when test = 'substring-after($guid-value,"uid:") = ""'>
      <xsl:attribute name = "R">
        <xsl:value-of select = 'substring-before($guid-value,"::")'/>
      </xsl:attribute>
	</xsl:when>
	<xsl:otherwise>
	  <!-- uid: reference -->
      <xsl:attribute name = "R">
        <xsl:value-of select = 'concat("uid:",//PortInstance[@Guid=substring-after($guid-value,"uid:")]/../../@Guid)'/>
      </xsl:attribute>
	</xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- computes connector end rPort reference from rConnection -->
<xsl:template name = "compute-connector-port-guid">
  <xsl:param name = "guid-value"> none </xsl:param>
  <xsl:choose>
    <!-- check, if we have ref: reference -->
    <xsl:when test = 'substring-after($guid-value,"uid:") = ""'>
	  <rPort>
	    <xsl:attribute name = "R">
          <xsl:value-of select = 'concat("ref:",substring-after($guid-value,"::"))'/>
        </xsl:attribute>
	  </rPort>
	</xsl:when>
    <xsl:otherwise>
	  <xsl:choose>
	    <!-- Really bad case - there is PortInstance, but its rType is not set (incorrect model) -->
	    <xsl:when test = 'count(//PortInstance[@Guid=substring-after($guid-value,"uid:")]/rType) &lt; 1'>
		  <!-- Just do nothing... -->
		</xsl:when>
		<xsl:otherwise>
          <rPort>
	        <xsl:attribute name = "R">
              <xsl:value-of select = '//PortInstance[@Guid=substring-after($guid-value,"uid:")]/rType/@R'/>
            </xsl:attribute>
          </rPort>
		</xsl:otherwise>
	  </xsl:choose>
	</xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- Switch reference from PortInstance to corresponding port in PortSymbol -->
<xsl:template match = "PortSymbol/rDefinition">
  <xsl:call-template name = "compute-portsymbol-port-guid">
    <xsl:with-param name = "guid-value"><xsl:value-of select = "@R"/></xsl:with-param>
  </xsl:call-template>
</xsl:template>

<!-- computes port reference from PortSymbol:rDefinition -->
<xsl:template name = "compute-portsymbol-port-guid">
  <xsl:param name = "guid-value"> none </xsl:param>
  <xsl:choose>
	<!-- For OM 1.8 to 2.1 conversion - in OM 1.8, this reference was set to Port (same as OM 2.1), while in OM 2.0 and 1.8.3 - to PortInstance -->
    <xsl:when test = 'count(//PortInstance[@Guid=substring-after($guid-value,"uid:")]) &lt; 1'>
	  <rDefinition>
	    <xsl:attribute name = "R">
	      <xsl:value-of select = '$guid-value'/>
	    </xsl:attribute>
	  </rDefinition>
	</xsl:when>
	<xsl:otherwise>
	  <xsl:choose>
	    <!-- Really bad case - there is PortInstance, but its rType is not set (incorrect model) -->
		<xsl:when test = 'count(//PortInstance[@Guid=substring-after($guid-value,"uid:")]/rType) &lt; 1'>
		  <!-- Just do nothing... -->
		</xsl:when>
		<xsl:otherwise>
		  <rDefinition>
		    <xsl:attribute name = "R">
	          <xsl:choose>
		        <!-- ref: reference - need to compute actual guid, since ref: is not permitted in links from presentation model -->
		        <xsl:when test = 'substring-after(//PortInstance[@Guid=substring-after($guid-value,"uid:")]/rType/@R,"uid:") = ""'>
	              <xsl:value-of select = 'concat("uid:",substring-before(substring-after(//PortInstance[@Guid=substring-after($guid-value,"uid:")]/rType/@R,"ref:#r"),"#"))'/>
		        </xsl:when>
		        <!-- uid: reference -->
		        <xsl:otherwise>
	              <xsl:value-of select = '//PortInstance[@Guid=substring-after($guid-value,"uid:")]/rType/@R'/>
		        </xsl:otherwise>
		      </xsl:choose>
            </xsl:attribute>
		  </rDefinition>
	    </xsl:otherwise>
	  </xsl:choose>
	</xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- value attributes now represented as strings -->
<xsl:template match = "IntegerValue | RealValue | HexstringValue | BitstringValue | CharstringValue | CharacterValue">
  <xsl:copy>
    <xsl:for-each select="@*">
	  <xsl:choose>
		<xsl:when test="local-name()='Value'">
		  <xsl:attribute name="ValueString"><xsl:value-of select="../@Value"/></xsl:attribute>
		</xsl:when>
		<xsl:otherwise>
		  <xsl:copy/>
		</xsl:otherwise>
	  </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- remove CommentLine::attachedTo -->
<xsl:template match = "CommentLine/lAttachedTo">
</xsl:template>

<!-- Symbol mlCommentLine link (reverse link for AttachedTo) -->
<xsl:template match = "lCommentLine">
  <rCommentLine>
	<xsl:call-template name="link-to-reference"/>
  </rCommentLine>
</xsl:template>

<!-- remove ModelElement::Stereotype -->
<xsl:template match = "rStereotype">
</xsl:template>

<!-- remove Transition::txOnException -->
<xsl:template match = "ExceptionTransition | LabelTransition | TriggeredTransition | StartTransition">
  <xsl:copy>
    <xsl:for-each select="@*">
	  <xsl:choose>
		<xsl:when test="local-name()='txOnException'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:otherwise>
		  <xsl:copy/>
		</xsl:otherwise>
	  </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- remove Expression::syntype -->
<xsl:template match = "rSyntype">
</xsl:template>

<!-- Introduced Parameter class -->

<!-- Parameters in EventClasses - rename Attributes to Parameters and remove not supported attributes -->
<xsl:template match = "cParameter/Attribute">
  <Parameter>
    <xsl:for-each select="@*">
	  <xsl:choose>
		<xsl:when test="local-name()= 'OwnerScope'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:when test="local-name()= 'Visibility'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:when test="local-name()= 'TargetScope'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:otherwise>
		  <xsl:copy/>
		</xsl:otherwise>
	  </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </Parameter>
</xsl:template>

<!-- remove not supported descendants in Parameters -->
<xsl:template match = "cParameter/Attribute/cInitialCount">
</xsl:template>

<xsl:template match = "Attribute/cPortInstance">
</xsl:template>

<xsl:template match = "PortSymbol/lPortInstance">
</xsl:template>

<!-- remove Attribute::Direction -->
<xsl:template match = "Attribute">
  <xsl:copy>
    <xsl:for-each select="@*">
	  <xsl:choose>
		<xsl:when test="local-name()='Direction'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:otherwise>
		  <xsl:copy/>
		</xsl:otherwise>
	  </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- Remove all Definition attributes/descendants from Generalization and Dependency (Relationship no longer inherits Definition) -->
<!-- attributes -->
<xsl:template match = "Generalization | Dependency">
  <xsl:copy>
    <xsl:for-each select="@*">
	  <xsl:choose>
		<xsl:when test="local-name()='Name'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:when test="local-name()='External'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:otherwise>
		  <xsl:copy/>
		</xsl:otherwise>
	  </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- descendants -->
<xsl:template match = "Generalization/cClientDependency | Dependency/cClientDependency">
</xsl:template>

<!-- change link TextSymbol::Definition to a reference and rename it to ShownEntities -->
<xsl:template match = "TextSymbol/lDefinition">
  <rShownEntities>
	<xsl:call-template name="link-to-reference"/>
  </rShownEntities>
</xsl:template>

<!-- remove the reverse link from Definition to TextSymbol -->
<xsl:template match = "lTextSymbol">
</xsl:template>

<!-- remove Definition::rPlace -->
<xsl:template match = "rPlace">
</xsl:template>

<!-- remove derived attribute from definition, which no longer support it -->
<xsl:template match = "BuildTask | ConnectionPoint | ExceptionHandler |
					   State | InformalDefinition | Label | DataType | Choice | Class |
					   Interface | Stereotype | Syntype | Exception | Signal | Timer | Port |
					   SignalList | AttributeTemplateParameter | EventTemplateParameter |
					   TypeTemplateParameter | ValueTemplateParameter">
  <xsl:copy>
    <xsl:for-each select="@*">
	  <xsl:choose>
		<xsl:when test="local-name()='Derived'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:otherwise>
		  <xsl:copy/>
		</xsl:otherwise>
	  </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- remove InteractionFragment::actualGate -->
<xsl:template match="InteractionFragment/cActualGate">
</xsl:template>

<!-- Remove SDL-based model for parallel state machines -->
<xsl:template match = "ConnectionPointInstance | StatePartition | StateAggregation | 
					   StateAggregationDiagram | StatePartitionSymbol">
</xsl:template>

<!-- Replace the link between MultiState and State with a reference from MultiState to State -->
<xsl:template match = "MultiState/lIncluded">
  <rIncluded>
	<xsl:call-template name="link-to-reference"/>
  </rIncluded>
</xsl:template>

<!-- remove reverse link from State to MultiState -->
<xsl:template match = "State/lIncluding">
</xsl:template>

<!-- replace AssignmentAction with ExpressionAction with AssignmentExpr -->
<xsl:template match = "AssignmentAction">
  <ExpressionAction>
	<xsl:call-template name="copy-all-attributes"/>
    <cExpression>
	  <AssignmentExpr>
	    <xsl:apply-templates/>
	  </AssignmentExpr>
    </cExpression>
  </ExpressionAction>
</xsl:template>

<xsl:template match = "AssignmentAction/cLhs">
  <cLeftOperand>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </cLeftOperand>
</xsl:template>

<xsl:template match = "AssignmentAction/cRhs">
  <cRightOperand>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </cRightOperand>
</xsl:template>

<!-- remove BehaviorSymbol -->
<xsl:template match = "BehaviourSymbol">
</xsl:template>

<!-- SignatureSymbol::AttributeLabel stuff -->
<!-- Replace SignatureSymbol::FeatureLabel with several more specific compositions -->
<!-- Event classes -->
<xsl:template match = "ExceptionSymbol/cAttributeLabel | OperationSymbol/cAttributeLabel |
					   StateMachineSymbol/cAttributeLabel | SignalSymbol/cAttributeLabel |
					   TimerSymbol/cAttributeLabel">
  <cParameterLabel>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </cParameterLabel>
</xsl:template>

<xsl:template match = "ExceptionSymbol/cAttributeLabel/FeatureLabel | 
					   OperationSymbol/cAttributeLabel/FeatureLabel |
					   StateMachineSymbol/cAttributeLabel/FeatureLabel |
					   SignalSymbol/cAttributeLabel/FeatureLabel |
					   TimerSymbol/cAttributeLabel/FeatureLabel">
  <ParameterLabel>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </ParameterLabel>
</xsl:template>

<!-- all from Attribute label of DataType - to OperationLabel -->
<xsl:template match = "DataTypeSymbol/cAttributeLabel">
  <cOperationLabel>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </cOperationLabel>
</xsl:template>

<xsl:template match = "DataTypeSymbol/cAttributeLabel/FeatureLabel">
  <OperationLabel>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </OperationLabel>
</xsl:template>

<!-- Attribute labels in Stuctured classifiers - just rename them -->
<xsl:template match = "ClassSymbol/cAttributeLabel/FeatureLabel |
					   InterfaceSymbol/cAttributeLabel/FeatureLabel | 
					   StereotypeSymbol/cAttributeLabel/FeatureLabel">
  <AttributeLabel>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </AttributeLabel>
</xsl:template>

<xsl:template match = "ClassSymbol/cOperationLabel/FeatureLabel | 
					   InterfaceSymbol/cOperationLabel/FeatureLabel |
					   StereotypeSymbol/cOperationLabel/FeatureLabel">
  <OperationLabel>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </OperationLabel>
</xsl:template>

<!-- rename UseCaseCollaborationDiagram to UseCaseDiagram -->
<xsl:template match = "UseCaseCollaborationDiagram">
  <UseCaseDiagram>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </UseCaseDiagram>
</xsl:template>    

<!-- remove FlowLine::joinAction -->
<xsl:template match = "FlowLine/lJoinAction">
</xsl:template>

<xsl:template match = "JoinAction/lFlowLine">
</xsl:template>

<!-- remove TextContainer features from those symbols, which no longer inherit it -->
<xsl:template match = "MethodSuspensionSymbol | DestroySymbol | MethodActivationSymbol">
  <xsl:copy>
    <xsl:for-each select="@*">
	  <xsl:choose>
		<xsl:when test="local-name()='Text'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:when test="local-name()='IsParseError'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:otherwise>
		  <xsl:copy/>
		</xsl:otherwise>
	  </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- rename DecisionAnswer to DecisionAnswerTransition -->
<xsl:template match = "DecisionAnswer">
  <DecisionAnswerTransition>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </DecisionAnswerTransition>
</xsl:template>

<!-- rename DecisionAnswer/cAnswer to cAction (since now DecisionAnswer is a transition) -->
<xsl:template match = "DecisionAnswer/cAnswer">
  <cAction>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </cAction>
</xsl:template>

<!-- remove ExtendLine and IncludeLine, use DependencyLine instead -->
<xsl:template match = "ExtendLine | IncludeLine">
  <DependencyLine>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </DependencyLine>
</xsl:template>

<xsl:template match = "ExtendLine/lDependency | IncludeLine/lDependency">
  <rRelationship>
	<xsl:call-template name="link-to-reference"/>
  </rRelationship>
</xsl:template>

<!-- remove reverse links in Dependency to Extend/Include lines -->
<xsl:template match = "Dependency/lIncludeLine | Dependency/lExtendLine">
</xsl:template>

<!-- remove Stereotype::metaClassList -->
<xsl:template match = "Stereotype">
  <xsl:copy>
    <xsl:for-each select="@*">
	  <xsl:choose>
		<xsl:when test="local-name()='MetaClasses'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:otherwise>
		  <xsl:copy/>
		</xsl:otherwise>
	  </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- remove derived attribute from definition, which no longer support it, -->
<!-- and move BuildArtifacts from Package to top level -->
<xsl:template match = "Package">
  <xsl:for-each select = "cBuildArtifact/BuildArtifact">
    <BuildObject>
      <xsl:for-each select="@*">
	    <xsl:choose>
	  	  <xsl:when test="local-name()='Derived'">
          <!-- do not copy this attribute -->
		  </xsl:when>
		  <xsl:otherwise>
		    <xsl:copy/>
		  </xsl:otherwise>
	    </xsl:choose>
      </xsl:for-each>
      <xsl:apply-templates/>
    </BuildObject>
  </xsl:for-each>
  <xsl:copy>
    <xsl:for-each select="@*">
	  <xsl:choose>
		<xsl:when test="local-name()='Derived'">
        	<!-- do not copy this attribute -->
		</xsl:when>
		<xsl:otherwise>
		  <xsl:copy/>
		</xsl:otherwise>
	  </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template match = "Package/cBuildArtifact">
</xsl:template>

<xsl:template match = "BuildArtifact/cStereotypeInstance">
</xsl:template>

<!-- remove MultiExceptionHandler::text -->
<xsl:template match = "MultiExceptionHandler">
  <xsl:copy>
    <xsl:for-each select="@*">
	  <xsl:choose>
		<xsl:when test="local-name()='Text'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:otherwise>
		  <xsl:copy/>
		</xsl:otherwise>
	  </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- remove BooleanValue - transform them to Idents -->
<xsl:template match = "BooleanValue">
  <Ident>
	<xsl:if test = "count(@Value) &lt; 1">
	  <xsl:attribute name="Name">false</xsl:attribute>
	</xsl:if>
    <xsl:for-each select="@*">
	  <xsl:choose>
		<xsl:when test="local-name()='Value'">
		  <xsl:attribute name="Name"><xsl:value-of select="../@Value"/></xsl:attribute>
		</xsl:when>
		<xsl:otherwise>
		  <xsl:copy/>
		</xsl:otherwise>
	  </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </Ident>
</xsl:template>

<!-- Remove links from the model -->
<xsl:template match = "ConnectionPoint/lConnectionPointSymbol | lTransitionLine">
</xsl:template>

<xsl:template match = "ConnectionPointSymbol/lConnectionPoint">
  <rConnectionPoint>
	<xsl:call-template name="link-to-reference"/>
  </rConnectionPoint>
</xsl:template>

<xsl:template match = "TransitionLine/lTransition">
  <rTransition>
	<xsl:call-template name="link-to-reference"/>
  </rTransition>
</xsl:template>

<xsl:template match = "StartTransition/lMultiState | ExceptionTransition/lMultiState | 
					   TriggeredTransition/lMultiState | LabelTransition/lMultiState |
					   Save/lMultiState">
</xsl:template>

<xsl:template match = "MultiState/lTransition">
  <rTransition>
	<xsl:call-template name="link-to-reference"/>
  </rTransition>
</xsl:template>

<xsl:template match = "MultiState/lSave">
  <rSave>
	<xsl:call-template name="link-to-reference"/>
  </rSave>
</xsl:template>

<xsl:template match = "LabelTransition/lJunctionSymbol">
</xsl:template>

<xsl:template match = "JunctionSymbol/lLabelTransition">
  <rLabelTransition>
	<xsl:call-template name="link-to-reference"/>
  </rLabelTransition>
</xsl:template>

<xsl:template match = "ExceptionTransition/lMultiExceptionHandler">
</xsl:template>

<xsl:template match = "MultiExceptionHandler/lExceptionTransition">
  <rExceptionTransition>
	<xsl:call-template name="link-to-reference"/>
  </rExceptionTransition>
</xsl:template>

<xsl:template match = "StartTransition/lStartSymbol">
</xsl:template>

<xsl:template match = "StartSymbol/lStartTransition">
  <rStartTransition>
	<xsl:call-template name="link-to-reference"/>
  </rStartTransition>
</xsl:template>
    
<xsl:template match = "Save/lSaveSymbol">
</xsl:template>

<xsl:template match = "SaveSymbol/lSave">
  <rSave>
	<xsl:call-template name="link-to-reference"/>
  </rSave>
</xsl:template>

<xsl:template match = "DecisionAnswer/lDecisionAnswerSymbol">
</xsl:template>

<xsl:template match = "DecisionAnswerSymbol/lDecisionAnswer">
  <rDecisionAnswer>
	<xsl:call-template name="link-to-reference"/>
  </rDecisionAnswer>
</xsl:template>

<xsl:template match = "TriggeredTransition/lInputSymbol | TriggeredTransition/lGuardSymbol">
</xsl:template>

<xsl:template match = "InputSymbol/lTriggeredTransition">
  <rTriggeredTransition>
	<xsl:call-template name="link-to-reference"/>
  </rTriggeredTransition>
</xsl:template>

<xsl:template match = "GuardSymbol/lTriggeredTransition">
  <rTriggeredTransition>
	<xsl:call-template name="link-to-reference"/>
  </rTriggeredTransition>
</xsl:template>

<xsl:template match = "CompoundAction/lTaskSymbol">
</xsl:template>

<xsl:template match = "TaskSymbol/lCompoundAction">
  <rCompoundAction>
	<xsl:call-template name="link-to-reference"/>
  </rCompoundAction>
</xsl:template>

<xsl:template match = "DecisionAction/lDecisionSymbol">
</xsl:template>

<xsl:template match = "DecisionSymbol/lDecisionAction">
  <rDecisionAction>
	<xsl:call-template name="link-to-reference"/>
  </rDecisionAction>
</xsl:template>

<xsl:template match = "OutputAction/lOutputSymbol">
</xsl:template>

<xsl:template match = "OutputSymbol/lOutputAction">
  <rOutputAction>
	<xsl:call-template name="link-to-reference"/>
  </rOutputAction>
</xsl:template>

<xsl:template match = "ReturnAction/lReturnSymbol">
</xsl:template>

<xsl:template match = "ReturnSymbol/lReturnAction">
  <rReturnAction>
	<xsl:call-template name="link-to-reference"/>
  </rReturnAction>
</xsl:template>

<xsl:template match = "StopAction/lStopSymbol">
</xsl:template>

<xsl:template match = "StopSymbol/lStopAction">
  <rStopAction>
	<xsl:call-template name="link-to-reference"/>
  </rStopAction>
</xsl:template>

<xsl:template match = "MultiState/lStateSymbol">
</xsl:template>

<xsl:template match = "StateSymbol/lMultiState">
  <rMultiState>
	<xsl:call-template name="link-to-reference"/>
  </rMultiState>
</xsl:template>

<xsl:template match = "Interaction/lInteractionSymbol">
</xsl:template>

<xsl:template match = "InteractionSymbol/lInteraction">
  <rInteraction>
	<xsl:call-template name="link-to-reference"/>
  </rInteraction>
</xsl:template>

<xsl:template match = "UseCase/lSubject">
</xsl:template>

<xsl:template match = "Subject/lUseCase">
  <rUseCase>
	<xsl:call-template name="link-to-reference"/>
  </rUseCase>
</xsl:template>

<xsl:template match = "Performance/lPerformanceLine">
</xsl:template>

<xsl:template match = "PerformanceLine/lPerformance">
  <rPerformance>
	<xsl:call-template name="link-to-reference"/>
  </rPerformance>
</xsl:template>

<xsl:template match = "Comment/lCommentSymbol">
</xsl:template>

<xsl:template match = "CommentSymbol/lDisplayedComment">
  <rDisplayedComment>
	<xsl:call-template name="link-to-reference"/>
  </rDisplayedComment>
</xsl:template>

<!-- ExceptionHandler mlOutgoing link -->
<xsl:template match = "ExceptionTransition/lFromHandler">
</xsl:template>

<xsl:template match = "ExceptionHandler/lOutgoing">
  <rOutgoing>
	<xsl:call-template name="link-to-reference"/>
  </rOutgoing>
</xsl:template>

<!-- Line Src/Dst transformation -->
<xsl:template match = "CommentLine/lSrc | ConnectorLine/lSrc | FlowLine/lSrc |
					   AssociationLine/lSrc | DependencyLine/lSrc | GeneralizationLine/lSrc |
					   TransitionLine/lSrc">
  <rSrc>
	<xsl:call-template name="link-to-reference"/>
  </rSrc>
</xsl:template>

<xsl:template match = "CommentLine/lDst | ConnectorLine/lDst | FlowLine/lDst |
					   AssociationLine/lDst | DependencyLine/lDst | GeneralizationLine/lDst |
					   TransitionLine/lDst">
  <rDst>
	<xsl:call-template name="link-to-reference"/>
  </rDst>
</xsl:template>

<!-- DiagramElement mlOutgoing link -->
<xsl:template match = "lOutgoing">
</xsl:template>

<!-- DiagramElement mlIncoming link -->
<xsl:template match = "lIncoming">
</xsl:template>

<xsl:template match = "MultiExceptionHandler/lExceptionHandler">
  <rExceptionHandler>
	<xsl:call-template name="link-to-reference"/>
  </rExceptionHandler>
</xsl:template>

<xsl:template match = "ExceptionHandler/lMultiExceptionHandler">
</xsl:template>

<xsl:template match = "Operation | StateMachine">
  <xsl:copy>
    <xsl:for-each select="@*">
	  <xsl:choose>
		<xsl:when test="local-name()='OperationKind'">
        <!-- do not copy this attribute -->
		</xsl:when>
		<xsl:otherwise>
		  <xsl:copy/>
		</xsl:otherwise>
	  </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<!-- Default rule -->
<xsl:template match="*">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template name="copy-all-attributes">
  <xsl:for-each select="@*">
    <xsl:copy/>
  </xsl:for-each>
</xsl:template>

<!-- template which renames attributes of link to attributes of reference -->
<xsl:template name = "link-to-reference">
  <xsl:for-each select="@*">
	<xsl:choose>
	  <xsl:when test="local-name()='L'">
	    <xsl:attribute name="R"><xsl:value-of select="../@L"/></xsl:attribute>
	  </xsl:when>
	  <xsl:otherwise>
	    <xsl:copy/>
	  </xsl:otherwise>
    </xsl:choose>
  </xsl:for-each>
  <xsl:apply-templates/>
</xsl:template>

</xsl:transform>