<!-- ***************************************** -->
<!-- XSLT transformation for converting from   -->
<!-- U2 XML storage format 2.9 (4) to 3.0 (4)  -->
<!-- ***************************************** -->

<xsl:transform version="1.1"
   xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:u2="http://www.telelogic.com/tau"
   exclude-result-prefixes='u2'>

<!-- Change file format version number to 3.0 -->
<xsl:template match="u2 | u2:u2">
  <xsl:copy>
    <xsl:for-each select="@*">
      <xsl:choose>
        <xsl:when test="local-name()='OMVersion'">
          <xsl:attribute name="OMVersion">3.0</xsl:attribute>
        </xsl:when>
        <xsl:when test="local-name()='StorageVersion'">
          <xsl:attribute name="StorageVersion">4</xsl:attribute>
        </xsl:when>
        <xsl:otherwise>
          <xsl:copy/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:for-each>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>


<!-- Default rule -->
<xsl:template match="*">
  <xsl:copy>
    <xsl:call-template name="copy-all-attributes"/>
    <xsl:apply-templates/>
  </xsl:copy>
</xsl:template>

<xsl:template name="copy-all-attributes">
  <xsl:for-each select="@*">
    <xsl:copy/>
  </xsl:for-each>
</xsl:template>

<!-- InformalEntityFragment rule -->
<xsl:template match="InformalEntityFragment | u2:InformalEntityFragment">
  <xsl:choose>
    <xsl:when test="count(rDefinition) &gt; 0 or count(u2:rDefinition) &gt; 0">
      <xsl:for-each select="rDefinition/Ident | u2:rDefinition/u2:Ident">
        <xsl:copy>
          <xsl:call-template name="copy-all-attributes"/>
          <xsl:apply-templates/>
        </xsl:copy>
      </xsl:for-each>
    </xsl:when>
    <xsl:otherwise>
      <xsl:copy>
        <xsl:for-each select="@*">
          <xsl:if test="local-name()!='Guid'">
            <xsl:copy/>
          </xsl:if>
        </xsl:for-each>
        <xsl:apply-templates/>
      </xsl:copy>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- <<icon>> -> <<labelPosition>> stransformation -->
<xsl:template match="u2:cExpression[ count(u2:BinaryExpr[    string(u2:rOperation/u2:Ident/@Name) = '=' 
                                                         and string(u2:cLeftOperand/u2:Ident/@Name) = 'labelPosition'])
                                     > 0]"
              mode="icon-to-label-position1">
  <u2:InstanceExpr>
    <u2:rInstanceOf>
      <u2:Ident Name="labelPosition">
        <u2:rDefinition R="uid:@TTDStereotypeDetails@labelPosition"/>
      </u2:Ident>
    </u2:rInstanceOf>
    <u2:cExpression>
      <xsl:apply-templates select="u2:BinaryExpr[    string(u2:rOperation/u2:Ident/@Name) = '='
                                                 and string(u2:cLeftOperand/u2:Ident/@Name) = 'labelPosition']"/>
    </u2:cExpression>
  </u2:InstanceExpr>
</xsl:template>

<xsl:template match="u2:cExpression[ count(u2:BinaryExpr[    string(u2:rOperation/u2:Ident/@Name) = '=' 
                                                         and string(u2:cLeftOperand/u2:Ident/@Name) != 'labelPosition'])
                                     > 0]"
              mode="icon-to-label-position2">
  <u2:InstanceExpr>
    <u2:rInstanceOf>
      <u2:Ident Name="icon">
        <u2:rDefinition R="uid:@TTDStereotypeDetails@stereotypeDetails"/>
      </u2:Ident>
    </u2:rInstanceOf>
    <u2:cExpression>
      <xsl:apply-templates select="u2:BinaryExpr[    string(u2:rOperation/u2:Ident/@Name) = '='
                                                 and string(u2:cLeftOperand/u2:Ident/@Name) != 'labelPosition']"/>
    </u2:cExpression>
  </u2:InstanceExpr>
</xsl:template>

<xsl:template match="u2:InstanceExpr[count(u2:rInstanceOf/u2:Ident[@Name='icon']) = 1]">
  <xsl:apply-templates mode="icon-to-label-position1"/>
  <xsl:apply-templates mode="icon-to-label-position2"/>
</xsl:template>


</xsl:transform>