#include "t3rts.h"
#include "codec.h"

#include <stdlib.h>

static t3rt_codecs_result_t tlt_My_Piggibacked_Message_encode(t3rt_value_t value, 
                                                              t3rt_binary_string_t encoded_data, 
                                                              t3rt_context_t ctx);

static t3rt_codecs_result_t tlt_My_Piggibacked_Message_decode(t3rt_binary_string_iter_t * encoded_data, 
                                                              t3rt_type_t type, 
                                                              t3rt_alloc_strategy_t strategy, 
                                                              t3rt_value_t * decoded_value, 
                                                              t3rt_context_t ctx);


void tlt_codec_init_function(t3rt_context_t ctx)
{
	/* This function is called to initialize the codec. 
	 * Nothing needs to be done in this case */

	return;
}


void tlt_codec_setup_function(t3rt_type_t type, t3rt_context_t ctx)
{
	/* This function is called in a loop, once with every type of the
	 * test suite. Its task is to associate an encoder and/or a decoder
	 * with the type, if encoding and decoding for that type should be
	 * supported by the codec */

	const char *typename = t3rt_type_definition_name (type, ctx);

	if (strcmp (typename, "My_Piggibacked_Message") == 0) {
		t3rt_type_set_encoder (type, tlt_My_Piggibacked_Message_encode, ctx);
		t3rt_type_set_decoder (type, tlt_My_Piggibacked_Message_decode, ctx);
	}

}

static t3rt_codecs_result_t tlt_My_Piggibacked_Message_encode(t3rt_value_t value, 
                                                              t3rt_binary_string_t encoded_data, 
                                                              t3rt_context_t ctx)
{
	t3rt_value_t field_value;
	t3rt_type_t field_type;
	t3rt_encoder_function_t asn1_message_encoder;

	const char *str;

	/* Encode Header */	
	field_value = t3rt_value_field_by_index (value, 0, ctx);
	str = t3rt_value_get_string (field_value, ctx);
	t3rt_binary_string_append_1byte (encoded_data, strlen(str)+1, ctx);
	t3rt_binary_string_append_nbytes (encoded_data, str, strlen(str)+1, ctx);

	/* Encode My_Asn1_Message */
	field_value = t3rt_value_field_by_index (value, 1, ctx);
	field_type = t3rt_value_type (field_value, ctx);
	asn1_message_encoder = t3rt_type_get_encoder (field_type, ctx);
	
	return (*asn1_message_encoder)(field_value, encoded_data, ctx);
}	


static t3rt_codecs_result_t tlt_My_Piggibacked_Message_decode(t3rt_binary_string_iter_t * encoded_data, 
                                                              t3rt_type_t type, 
                                                              t3rt_alloc_strategy_t strategy, 
                                                              t3rt_value_t * decoded_value, 
                                                              t3rt_context_t ctx)
{
	int remaining;
	char bufflen;
	char *buffer, *buffend;
	t3rt_value_t field_value;
	t3rt_value_t decoded_asn1_value;
	t3rt_type_t field_type;
	t3rt_decoder_function_t asn1_message_decoder;
	t3rt_codecs_result_t result;
		
	*decoded_value = t3rt_type_instantiate_value(type, strategy, ctx);
	t3rt_value_set_vector_empty (*decoded_value, ctx);

	remaining = t3rt_bstring_iter_remaining_room(encoded_data, ctx) / 8;
	if (remaining < 2) {
		return t3rt_codecs_result_failed_c;
	}

	/* Header */
	bufflen = t3rt_bstring_iter_get_1byte (encoded_data, ctx);
	buffend = buffer = (char *)malloc (bufflen+1);
	for(;;) {
		unsigned char c = t3rt_bstring_iter_get_1byte(encoded_data, ctx);
		*(buffend++) = c;
		if (!c) break;
	}
	field_value = t3rt_value_field_by_index (*decoded_value, 0, ctx);
	t3rt_value_set_string (field_value, buffer, ctx);
	free (buffer);

	/* Octetstring */
	field_value = t3rt_value_field_by_index (*decoded_value, 1, ctx);
	field_type = t3rt_value_type (field_value, ctx);
	asn1_message_decoder = t3rt_type_get_decoder (field_type, ctx);

	result = (*asn1_message_decoder)(encoded_data, field_type, strategy, &decoded_asn1_value, ctx);
	if (result != t3rt_codecs_result_succeeded_c)
		return result;
	t3rt_value_assign_vector_element (*decoded_value, 1, decoded_asn1_value, ctx);
	return t3rt_codecs_result_succeeded_c;
}

/* End-Of-File */
