using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;

namespace TestExecutor
{
    public partial class TestCaseParameters : Form
    {
        private ParameterType[] _parTypes;
        public ParameterType[] ParTypes
        {
            get { return _parTypes; }
            set { _parTypes = value; }
        }

        private Parameter[] _parValues;
        public Parameter[] ParValues
        {
            get { return _parValues; }
            set { _parValues = value; }
        }

        private Label[] _parLabels;
        private MaskedTextBox[] _parEdits;

        public TestCaseParameters(ParameterType[] parameters)
        {
            _parTypes = parameters;

            // initalize value array
            _parValues = new Parameter[_parTypes.Length];
            for (int i = 0; i < parameters.Length; ++i)
            {
                _parValues[i] = new Parameter();

                _parValues[i].parameterName = parameters[i].parameterName;
                _parValues[i].parameterValue = null;
                _parValues[i].passingMode = parameters[i].passingMode;
            }

            // create arrays for controls 
            _parLabels = new Label[_parTypes.Length];
            _parEdits = new MaskedTextBox[_parTypes.Length];

            InitializeComponent();
        }

        protected virtual string GetLabelText(ParameterType parType)
        {
            // set label text as "ParName : ParType :="
            return (parType.parameterName + " : ") +
                   (parType.parameterType.moduleName != null ? parType.parameterType.moduleName + "." : "") +
                   (parType.parameterType.objectName + " :=");
        }

        private void CreateControls()
        {
            for (int i = 0; i < _parTypes.Length; ++i)
            {
                // create labels
                _parLabels[i] = new Label();
                _parLabels[i].Parent = this;
                _parLabels[i].Text = GetLabelText(_parTypes[i]);

                // create edits
                _parEdits[i] = new MaskedTextBox();
                _parEdits[i].Parent = this;

                // disable controls for "out" parameters
                if (_parTypes[i].passingMode.Name == "OUT-PAR")
                {
                    _parEdits[i].Text = "<OUT PARAMETER>";
                    _parEdits[i].Enabled = false;
                }
            }
        }

        // position s controls on the screen properly
        // (this function is called from OnResize event)
        protected virtual void PositionControls()
        {
            const int offset = 3;

            // calculate max label width
            int maxWidth = 0;
            for (int i = 0; i < _parLabels.Length; ++i)
                maxWidth = System.Math.Max(maxWidth, _parLabels[i].PreferredWidth);

            // set positions for all controls
            for (int i = 0; i < _parLabels.Length; ++i)
            {
                _parLabels[i].Width = maxWidth;
                _parLabels[i].Margin = new Padding(0);

                Point labelPos = new Point(offset, (i + 1) * _parLabels[i].Height);
                _parLabels[i].Location = labelPos;

                _parEdits[i].Height = _parLabels[i].Height;
                _parEdits[i].Width = this.ClientSize.Width - maxWidth - 2*offset;
                _parEdits[i].Margin = new Padding(0);

                Point editPos = new Point(maxWidth + offset, labelPos.Y);
                _parEdits[i].Location = editPos;
            }

            Point buttonPos = new Point((this.ClientSize.Width - RunButton.Width)/2, 
                (_parLabels.Length + 2) * _parLabels[0].Height);
            RunButton.Location = buttonPos;
        }

        private void TestCaseParameters_Load(object sender, EventArgs e)
        {
            // create labels and text edits
            CreateControls();

            // position controls properly
            PositionControls();

            this.Height = RunButton.Location.Y + RunButton.Width;
        }

        private void OnResize(object sender, EventArgs e)
        {
            PositionControls();

            this.MinimumSize = new Size(_parLabels[0].Width, RunButton.Location.Y + RunButton.Width);
        }

        // On closing copy values from edit controls to alue array
        // TODO: cancel event and jump to empty entry field if any?
        private void TestCaseParameters_FormClosing(object sender, FormClosingEventArgs e)
        {
            for (int i = 0; i < _parEdits.Length; ++i)
            {
                if (_parEdits[i].Enabled)
                    _parValues[i].parameterValue = _parEdits[i].Text;
            }
        }
    }
}