#include <stdio.h>

#ifdef _WIN32
#include <winsock.h>
#else
#include <sys/types.h>
#include <sys/socket.h>
#include <pthread.h>
#endif

#include "TciSoapAccessBinding.nsmap"

#define DEFAULT_SOAP_PORT 18083
#define SOAP_PORT_SWITCH "-soap_port"

/*
 * External functions implemented in serverImpl.c
*/
extern void TciWsInit();
extern void t3ws__FillTestSuiteCache(struct soap *soap);

/*
 * Asynс thread routine.
 * Started for each test mangeemnt request
*/
#ifdef _WIN32
#define THREAD_RETURN
static void ServeRequest(void *par)
#else
#define THREAD_RETURN NULL
static void* ServeRequest(void *par)
#endif
{
    struct soap *soap = (struct soap*)par;

    soap_serve(soap); // serve request

    soap_destroy(soap);
    soap_end(soap);
    soap_done(soap);

    free(soap); 

    return THREAD_RETURN;
}

/*
 * Main function
 *
*/
int main(int argc, char* argv[])
{
   struct soap soap; 
   int m, s;
   int i;
   int soap_port = DEFAULT_SOAP_PORT;
   char hostname[256];
#ifdef _WIN32
  WSADATA wsaData;
#endif

#ifdef _WIN32
   WSAStartup(MAKEWORD(2,2), &wsaData);
#endif      

   TciWsInit();

   if (!tciInit(argc, argv))
       return 1;

   for (i = 0; i < argc; ++i) {
     if (strcmp(argv[i], SOAP_PORT_SWITCH) == 0 && i < argc-1) {
       soap_port = atoi(argv[i+1]);
       break;
     }
   }

   gethostname(hostname, 256);

   fprintf(stdout, "*********************************************************************************\n");
   fprintf(stdout, "Test suite server has been started.\n");
   fprintf(stdout, "Listening test management requests on '%s:%d'\n", hostname, soap_port);
   fprintf(stdout, "Use '%s' switch to specify other port\n", SOAP_PORT_SWITCH);
   fprintf(stdout, "*********************************************************************************\n");
   fflush(stdout);

   soap_init(&soap); 

   t3ws__FillTestSuiteCache(&soap);

   m = soap_bind(&soap, NULL, DEFAULT_SOAP_PORT, 100);

   if (m < 0) 
      soap_print_fault(&soap, stderr); 
   else
   { 

#ifdef DEBUG
      fprintf(stderr, "Socket connection successful: master socket = %d\n", m); 
#endif

      for (i = 1; ; i++) 
      {
         struct soap *tsoap;
#ifdef _WIN32  
         unsigned int thread_id = 0;
#else
         pthread_t thread_id = 0;
#endif  

         s = soap_accept(&soap); 
         if (s < 0) 
         { 
            soap_print_fault(&soap, stderr);
            break; 
         } 

#ifdef DEBUG
         fprintf(stderr, "%d: accepted connection from IP=%d.%d.%d.%d socket=%d", i, 
            (soap.ip >> 24)&0xFF, (soap.ip >> 16)&0xFF, (soap.ip >> 8)&0xFF, soap.ip&0xFF, s); 
#endif

         tsoap = soap_copy(&soap); // make a safe copy 
         if (!tsoap) 
            break; 

         // serve request in separate thread using tsoap handle
#ifdef _WIN32
         if (-1 == (thread_id = _beginthread(&ServeRequest, 0, (void*)tsoap)))
#else
         if (0 != pthread_create(&thread_id, NULL, &ServeRequest, (void*)tsoap))
#endif
            break; 
      } 
   } 
   soap_done(&soap); // close master socket and detach environment 

   tciFinalize();

#ifdef _WIN32
   WSACleanup();
#endif

   return 0;
}
