/***************************************************************************************
 * TCI Codec encoder/decoder plugin routines
 *
 * 2010 IBM Corporation
 *
 * This file contains encoder and decoder plugin routines
 * as specified in Systems Tester codec interface
 *
 ***************************************************************************************/

#include "t3rts.h"

/*
 * codec_init
 *
 * TCI codec initialization function
 *
 * It's called once during ETS initialization
*/
 		
static void
codec_init(t3rt_context_t ctx)
{
}

/*
 * codec_setup
 *
 * TCI codec setup function
 *
 * It's called for every built-in and user-defined type in testsuite
 *
 * It sets up encoder and decoder function for a type.
 * This function sets t3rt_tci_encode and t3rt_tci_decode functions
 * as encoder and decoder.
 *
 * t3rt_tci_encode and t3rt_tci_decode provide gateway between 
 * TCI codec interface and IBM Rational proprietary interface.
 * These function will call tciEncode and tciDecoder.
*/

static void
codec_setup(t3rt_type_t type, t3rt_context_t ctx)
{
  switch (t3rt_type_kind(type, ctx)) {
    case t3rt_type_kind_integer:
    case t3rt_type_kind_float:
    case t3rt_type_kind_boolean:
    case t3rt_type_kind_objectidentifier:
    case t3rt_type_kind_verdicttype:
    case t3rt_type_kind_charstring:
    case t3rt_type_kind_bitstring:
    case t3rt_type_kind_hexstring:
    case t3rt_type_kind_octetstring:
    case t3rt_type_kind_universal_charstring:
    case t3rt_type_kind_char:
    case t3rt_type_kind_universal_char:
    case t3rt_type_kind_record:
    case t3rt_type_kind_set:
    case t3rt_type_kind_recordof:
    case t3rt_type_kind_setof:
    case t3rt_type_kind_union:
    case t3rt_type_kind_enumerated:
    case t3rt_type_kind_array:
    case t3rt_type_kind_address:
      t3rt_type_set_encoder(type, t3rt_tci_encode, ctx);
      t3rt_type_set_decoder(type, t3rt_tci_decode, ctx);
  }
}

/*
 * t3ud_register_codecs
 *
 * This function is called byu the runtime system during the ETS initialization
 * Here user may register all user-defined codecs by executing t3rt_codecs_register
 * function for every codec.
*/ 

void t3ud_register_codecs (int argc, char *argv[], t3rt_context_t ctx)
{
  t3rt_codecs_register(codec_init, codec_setup, ctx);
}
