/***************************************************************************************
 * TCI Codec encoder/decoder implementation (t3tci_codec.c)
 *
 * 2010 IBM Corporation
 *
 * This is a simple codec implementation for 'integer' and 'charstring' types.
 * Type of the encoded value is determined using passed decoding hypothesis.
 *
 ***************************************************************************************/

#include "tci.h"

BinaryString tciEncode(TciValue value)
{
  BinaryString msg;
  unsigned char *cur_pos = 0;

  msg.data = 0;
  msg.bits = 0;
  msg.aux = 0;
  
  switch(tciGetTypeClass(tciGetType(value))) 
  { 
    case TCI_CHARSTRING_TYPE:
      {
        TciCharStringValue cstr_value = tciGetCStringValue(value);
        
        msg.bits = (4 + cstr_value->length) * 8;
        msg.data = tciMemoryAllocate(msg.bits / 8);
        cur_pos = msg.data;
        
        memcpy(cur_pos, &cstr_value->length, 4);
        memcpy(cur_pos + 4, cstr_value->string, cstr_value->length);
      }
      break;

    case TCI_INTEGER_TYPE:
      {
        unsigned char sign = tciGetIntSign(value);
        String abs_str = tciGetIntAbs(value);
        unsigned long n_digits = strlen(abs_str); 
        
        msg.bits = (1 + 4 + n_digits) * 8;
        msg.data = tciMemoryAllocate(msg.bits / 8);
        cur_pos = msg.data;
        
        memcpy(cur_pos, &sign, 1);
        memcpy(cur_pos + 1, &n_digits, 4);
        memcpy(cur_pos + 1 + 4, abs_str, n_digits);
      }
      break;
    
    default:
      /* There is no need to decode other types (see ttcn3tciCD example)*/
      ;
  }

  return msg;
}

TciValue tciDecode(BinaryString msg, TciType decHypothesis)
{
  unsigned char *cur_pos = msg.data;
  TciValue value = 0;

  switch(tciGetTypeClass(decHypothesis)) 
  {
    case TCI_CHARSTRING_TYPE:
      {
        unsigned long n_chars = *((unsigned long *)cur_pos);
        unsigned long i;

        value = tciNewInstance(tciGetTciCharstringType());
        tciSetCStringLength(value, n_chars);

        cur_pos += 4;
        for (i = 0; i < n_chars; i++)
        {
          tciSetCStringCharValue(value, i, cur_pos[i]);
        }
      }
      break;

    case TCI_INTEGER_TYPE:
      {
        unsigned char sign = *cur_pos;
        unsigned long n_digits = *((unsigned long *)(cur_pos + 1));
        unsigned long i;

        value = tciNewInstance(tciGetIntegerType());
        tciSetIntSign(value, sign);
        tciSetIntNumberOfDigits(value, n_digits);

        cur_pos += 5;
        for (i = 0; i < n_digits; i++)
        {
          /* digits are stored in tciIntegerType in the reversed order! */
          tciSetIntDigit(value, i, cur_pos[n_digits - 1 - i]);
        }
      }
      break;

    default:
      /* There is no need to decode other types (see ttcn3tciCD example)*/
      ;
  }
  
  return value;
}  

