/*---------------------------------------------------------------------------
 * File Name: t3tci_template.c
 *
 * 2010 IBM Corporation
 * 
 * Description:
 *     This is a template with functions that have to be implemented for
 *     TCI adaptation.
 */

#include "tci.h"

#if 0

/* Value decode (in TriMessageType message,  in Type decodingHypothesis)	
 * This operations decodes message according to the encoding rules and returns a TTCN-3 value. 
 * The decodingHypothesis shall be used to determine whether the encoded value can be decoded. 
 * If an encoding rule is not self-sufficient, 
 * i.e. if the encoded message does not inherently contain its type decodingHypothesis shall be used. 
 * If the encoded value can be decoded without the decoding hypothesis, 
 * the distinct null value shall be returned if the type determined from the encoded message 
 * is not compatible with the decoding hypothesis.
*/


TciValue tciDecode(BinaryString message, 
                   TciType decHypothesis) {

  return 0;                   
}

/* TriMessageType encode (in Value value)	
 * Returns an encoded TriMessage according to the encoding rules.
*/

BinaryString tciEncode(TciValue value) {

  BinaryString result;
 
  result.bits = 0;
  return result;
}

#endif

/* void tciTestCaseStarted (in TciTestCaseIdType testCaseId, in TciParameterListType parameterList, in Tfloat timer)	
 * tciTestCaseStarted indicates to the TM that a test case with testCaseId has been started. 
 * It will not be distinguished whether the test case has been started explicitly 
 * using the required operation tciStartTestCase or implicitly while executing the control part.
 *
 * Constraint: Shall only be called after either the control part of the module or a test case has been started 
 * using the required operations tciStartControl or tciStartTestCase.
*/
void tciTestCaseStarted(TciTestCaseIdType testCaseId, 
                        TciParameterListType parameterList, 
                        double timer) {}

/* void tciTestCaseTerminated (in VerdictValue verdict, in TciParameterListType parameterlist)	
 * This operation will be called by the TE to indicate the test management 
 * that the test case that has been currently executed on the MTC has terminated 
 * and that the final verdict was verdict. 
 * On the invocation of a tciTestCaseTerminated operation all out and inout test case parameters contain Values. 
 * All in test case parameters contain the distinct value of null 
 * because they are only of relevance to the test case start but not in the reply to the call.
 *
 * Constraint: Shall only be called after either the control part of the module or a test cases 
 * has been started using the required operations tciStartControl or tciStartTestCase.
*/
void tciTestCaseTerminated(TciValue verdict, 
                           TciParameterListType parameterlist) {}

/* void tciControlTerminated()	
 * This operation will be called by the TE to indicate the test management that the control part 
 * of the selected module has just terminated execution.
 *
 * Constraint: Shall only be called when the module execution has been started using the tciStartControl operation.
*/

/*
   tciControlTerminated() is generated by TTCN-3 compiler if MAINFILE = yes is specifed in mcfg file.
   It's used to release mutex that blocks main() function till termination of the control part
   
   void tciControlTerminated ();	
*/

/* Value tciGetModulePar (in TciModuleParameterIdType parameterId)	
 * The management provides to the TE a Value for the indicated parameterId. 
 * Every call of tciGetModulePar() will return the same value throughout the execution 
 * of an explicitly started test case or throughout the execution of a control part 
 * If the management cannot provide a TTCN-3 value, the distinct null value shall be returned.
 *
 * Constraint: This operation shall be called whenever the TE needs to access the value of a module parameter. 
 * Every accessed module parameter will be resolved only once between a tciStartTestCase and tciTestCaseTerminated 
 * pair if a test case has been started explicitly or between a tciStartControl and tciControlTerminated pair 
 * if the control part of a module has been started.
*/
TciValue tciGetModulePar(TciModuleParameterIdType parameterId) {
  
  return 0;
}

/* void tciError(in String message)	
 * The TE indicates the occurrence of an unrecoverable error situation. 
 * message contains a reason phrase that might be communicated to the test system user. 
 * It is up to the test management to terminate execution of test cases or control parts if running. 
 * The test management has to take explicit measures to terminate test execution immediately.
 *
 * Constraint: Can be called at any time by the TE to indicate an unrecoverable error situation. 
 * This error situation could either be indicated by the CH or the CD or could occur within the TE.
*/
void tciError(String message) {}


/* tliTcExecute
 * 
 * Shall be called by TE to log the execute test case request.
*/

void tliTcExecute (String am,
                   long int ts,
                   String src,
                   long int line,
                   TriComponentId c,
                   TciTestCaseIdType tcId,
                   TciParameterListType pars,
                   TriTimerDuration dur) {}

/* tliTcStart
 * 
 * Shall be called by TE to log the start of a testcase. This event occurs before the testcase is started.
*/
                          
void tliTcStart (String am,
                 long int ts,
                 String src,
                 long int line,
                 TriComponentId c,
                 TciTestCaseIdType tcId,
                 TciParameterListType pars,
                 TriTimerDuration dur) {}

/* tliTcStop
 * 
 * Shall be called by TE to log the stop of a testcase.
*/
                        
void tliTcStop (String am,
                long int ts,
                String src,
                long int line,
                TriComponentId c) {}

/* tliTcStarted
 * 
 * Shall be called by TM to log the start of a testcase. This event occurs after the testcase was started.
*/
                       
void tliTcStarted (String am,
                   long int ts,
                   String src,
                   long int line,
                   TriComponentId c,
                   TciTestCaseIdType tcId,
                   TciParameterListType pars,
                   TriTimerDuration dur) {}

/* tliTcTerminated
 * 
 * Shall be called by TM to log the termination of a testcase. This event occurs after the testcase
 * terminated.
*/
                          
void tliTcTerminated (String am,
                      long int ts,
                      String src,
                      long int line,
                      TriComponentId c,
                      TciTestCaseIdType tcId,
                      TciParameterListType pars,
                      TciValue outcome) {}

/* tliCtrlStart
 * 
 * Shall be called by TE to log the start of the control part. This event occurs before the control is
 * started. If the control is not represented by a TRI component, 'c' is null.
*/
                             
void tliCtrlStart (String am,
                   long int ts,
                   String src,
                   long int line,
                   TriComponentId c) {}

/* tliCtrlStop
 * 
 * Shall be called by TE to log the stop of the control part. This event occurs before the control is
 * stopped. If the control is not represented by a TRI component, 'c' is null.
*/
                          
void tliCtrlStop (String am,
                  long int ts,
                  String src,
                  long int line,
                  TriComponentId c) {}

/* tliCtrlTerminated
 * 
 * Shall be called by TM to log the termination of the control part. This event occurs after the control
 * has terminated. If the control is not represented by a TRI component, 'c' is null.
*/
 
void tliCtrlTerminated (String am,
                        long int ts,
                        String src,
                        long int line,
                        TriComponentId c) {}

/* tliMSend_m
 * 
 * Shall be called by SA to log a unicast send operation. This event occurs after sending. This event is
 * used for logging the communication with the SUT.
*/
                               
void tliMSend_m (String am,
                 long int ts,
                 String src,
                 long int line,
                 TriComponentId c,
                 TriPortId atPort,
                 TriPortId toPort,
                 TciValue msgValue,
                 TriAddress address,
                 TriStatus encoderFailure,
                 TriMessage msg,
                 TriStatus transmissionFailure) {}

/* tliMSend_m_BC
 * 
 * Shall be called by SA to log a broadcast send operation. This event occurs after sending. This event is
 * used for logging the communication with the SUT.
*/
                        
void tliMSend_m_BC (String am,
                    long int ts,
                    String src,
                    long int line,
                    TriComponentId c,
                    TriPortId atPort,
                    TriPortId toPort,
                    TciValue msgValue,
                    TriStatus encoderFailure,
                    TriMessage msg,
                    TriStatus transmissionFailure) {}

/* tliMSend_m_MC
 * 
 * Shall be called by SA to log a multicast send operation. This event occurs after sending. This event is
 * used for logging the communication with the SUT.
*/
                           
void tliMSend_m_MC (String am,
                    long int ts,
                    String src,
                    long int line,
                    TriComponentId c,
                    TriPortId atPort,
                    TriPortId toPort,
                    TciValue msgValue,
                    TriAddressList addresses,
                    TriStatus encoderFailure,
                    TriMessage msg,
                    TriStatus transmissionFailure) {}

/* tliMSend_c
 * 
 * Shall be called by CH to log a unicast send operation. This event occurs after sending. This event is
 * used for logging the intercomponent communication.
*/
                           
void tliMSend_c (String am,
                 long int ts,
                 String src,
                 long int line,
                 TriComponentId c,
                 TriPortId atPort,
                 TriPortId toPort,
                 TciValue msgValue,
                 TriStatus transmissionFailure) {}

/* tliMSend_c_BC
 * 
 * Shall be called by CH to log a broadcast send operation. This event occurs after sending. This event is
 * used for logging the intercomponent communication.
*/
                        
void tliMSend_c_BC (String am,
                    long int ts,
                    String src,
                    long int line,
                    TriComponentId c,
                    TriPortId atPort,
                    TriPortIdList toPortList,
                    TciValue msgValue,
                    TriStatus transmissionFailure) {}

/* tliMSend_c_MC
 * 
 * Shall be called by CH to log a multicast send operation. This event occurs after sending. This event is
 * used for logging the intercomponent communication.
*/
                           
void tliMSend_c_MC (String am,
                    long int ts,
                    String src,
                    long int line,
                    TriComponentId c,
                    TriPortId atPort,
                    TriPortIdList toPortList,
                    TciValue msgValue,
                    TriStatus transmissionFailure) {}

/* tliMDetected_m
 * 
 * Shall be called by SA to log the enqueuing of a message. This event occurs after the message is
 * enqueued. This event is used for logging the communication with the SUT.
*/
                           
void tliMDetected_m (String am,
                     long int ts,
                     String src,
                     long int line,
                     TriComponentId c,
                     TriPortId atPort,
                     TriPortId fromPort,
                     TriMessage msg,
                     TriAddress address) {}

/* tliMDetected_c
 * 
 * Shall be called by CH to log the enqueuing of a message. This event occurs after the message is
 * enqueued. This event is used for logging the intercomponent communication.
*/
                            
void tliMDetected_c (String am,
                     long int ts,
                     String src,
                     long int line,
                     TriComponentId c,
                     TriPortId atPort,
                     TriPortId fromPort,
                     TciValue msgValue) {}

/* tliMMismatch_m
 * 
 * Shall be called by TE to log the mismatch of a template. This event occurs after checking a
 * template match. This event is used for logging the communication with the SUT.
*/
                            
void tliMMismatch_m (String am,
                     long int ts,
                     String src,
                     long int line,
                     TriComponentId c,
                     TriPortId port,
                     TciValue msgValue,
                     TciValueTemplate msgTmpl,
                     TciValueDifferenceList diffs,
                     TriAddress address,
                     TciValueTemplate addressTmpl) {}

/* tliMMismatch_c
 * 
 * Shall be called by TE to log the mismatch of a template. This event occurs after checking a
 * template match. This event is used for logging the intercomponent communication.
*/

void tliMMismatch_c (String am,
                     long int ts,
                     String src,
                     long int line,
                     TriComponentId c,
                     TriPortId port,
                     TciValue msgValue,
                     TciValueTemplate msgTmpl,
                     TciValueDifferenceList diffs,
                     TriComponentId from,
                     TciNonValueTemplate fromTmpl) {}

/* tliMReceive_m
 * 
 * Shall be called by TE to log the receive of a message. This event occurs after checking a template
 * match. This event is used for logging the communication with SUT.
*/

void tliMReceive_m (String am,
                    long int ts,
                    String src,
                    long int line,
                    TriComponentId c,
                    TriPortId port,
                    TciValue msgValue,
                    TciValueTemplate msgTmpl,
                    TriAddress address,
                    TciValueTemplate addressTmpl) {}

/* tliMReceive_c
 * 
 * Shall be called by TE to log the receive of a message. This event occurs after checking a template
 * match. This event is used for logging the intercomponent communication.
*/
                           
void tliMReceive_c (String am,
                    long int ts,
                    String src,
                    long int line,
                    TriComponentId c,
                    TriPortId port,
                    TciValue msgValue,
                    TciValueTemplate msgTmpl,
                    TriComponentId from,
                    TciNonValueTemplate fromTmpl) {}

/* tliPrCall_m
 * 
 * Shall be called by SA to log a unicast call operation. This event occurs after call execution. This
 * event is used for logging the communication with the SUT.
*/
                           
void tliPrCall_m (String am,
                  long int ts,
                  String src,
                  long int line,
                  TriComponentId c,
                  TriPortId atPort,
                  TriPortId toPort,
                  TriSignatureId signature,
                  TciParameterListType parsValue,
                  TriAddress address,
                  TriStatus encoderFailure,
                  TriParameterList pars,
                  TriStatus transmissionFailure) {}

/* tliPrCall_m_BC
 * 
 * Shall be called by SA to log a broadcast call operation. This event occurs after call execution. This
 * event is used for logging the communication with the SUT.
*/
                         
void tliPrCall_m_BC (String am,
                     long int ts,
                     String src,
                     long int line,
                     TriComponentId c,
                     TriPortId atPort,
                     TriPortId toPort,
                     TriSignatureId signature,
                     TciParameterListType parsValue,
                     TriStatus encoderFailure,
                     TriParameterList pars,
                     TriStatus transmissionFailure) {}
                            
/* tliPrCall_m_MC
 * 
 * Shall be called by SA to log a multicast call operation. This event occurs after call execution. This
 * event is used for logging the communication with the SUT.
*/

void tliPrCall_m_MC (String am,
                     long int ts,
                     String src,
                     long int line,
                     TriComponentId c,
                     TriPortId atPort,
                     TriPortId toPort,
                     TriSignatureId signature,
                     TciParameterListType parsValue,
                     TriAddressList addresses,
                     TriStatus encoderFailure,
                     TriParameterList pars,
                     TriStatus transmissionFailure) {}

/* tliPrCall_c
 * 
 * Shall be called by CH to log a unicast call operation. This event occurs after call execution. This
 * event is used for logging the intercomponent communication.
*/
                            
void tliPrCall_c (String am,
                  long int ts,
                  String src,
                  long int line,
                  TriComponentId c,
                  TriPortId atPort,
                  TriPortId toPort,
                  TriSignatureId signature,
                  TciParameterListType parsValue,
                  TriStatus transmissionFailure) {}

/* tliPrCall_c_BC
 * 
 * Shall be called by CH to log a broadcast call operation. This event occurs after call execution. This
 * event is used for logging the intercomponent communication.
*/
                         
void tliPrCall_c_BC (String am,
                     long int ts,
                     String src,
                     long int line,
                     TriComponentId c,
                     TriPortId atPort,
                     TriPortIdList toPortList,
                     TriSignatureId signature,
                     TciParameterListType parsValue,
                     TriStatus transmissionFailure) {}

/* tliPrCall_c_MC
 * 
 * Shall be called by CH to log a multicast call operation. This event occurs after call execution. This
 * event is used for logging the intercomponent communication.
*/
                            
void tliPrCall_c_MC (String am,
                     long int ts,
                     String src,
                     long int line,
                     TriComponentId c,
                     TriPortId atPort,
                     TriPortIdList toPortList,
                     TriSignatureId signature,
                     TciParameterListType parsValue,
                     TriStatus transmissionFailure) {}

/* tliPrGetCallDetected_m
 * 
 * Shall be called by SA to log the getcall enqueue operation. This event occurs after call is enqueued.
 * This event is used for logging the communication with the SUT.
*/
                            
void tliPrGetCallDetected_m (String am,
                             long int ts,
                             String src,
                             long int line,
                             TriComponentId c,
                             TriPortId atPort,
                             TriPortId fromPort,
                             TriSignatureId signature,
                             TriParameterList pars,
                             TriAddress address) {}

/* tliPrGetCallDetected_c
 * 
 * Shall be called by CH to log the getcall enqueue operation. This event occurs after call is enqueued.
 * This event is used for logging the intercomponent communication.
*/
                                    
void tliPrGetCallDetected_c (String am,
                             long int ts,
                             String src,
                             long int line,
                             TriComponentId c,
                             TriPortId atPort,
                             TriPortId fromPort,
                             TriSignatureId signature,
                             TciParameterListType parsValue) {}

/* tliPrGetCallMismatch_m
 * 
 * Shall be called by TE to log the mismatch of a getcall. This event occurs after getcall is checked
 * against a template. This event is used for logging the communication with the SUT.
*/
                                    
void tliPrGetCallMismatch_m (String am,
                             long int ts,
                             String src,
                             long int line,
                             TriComponentId c,
                             TriPortId port,
                             TriSignatureId signature,
                             TciParameterListType parsValue,
                             TciValueTemplate parsTmpl,
                             TciValueDifferenceList diffs,
                             TriAddress address,
                             TciValueTemplate addressTmpl) {}

/* tliPrGetCallMismatch_c
 * 
 * Shall be called by TE to log the mismatch of a getcall. This event occurs after getcall is checked
 * against a template. This event is used for logging the intercomponent communication.
*/
                                    
void tliPrGetCallMismatch_c (String am,
                             long int ts,
                             String src,
                             long int line,
                             TriComponentId c,
                             TriPortId port,
                             TriSignatureId signature,
                             TciParameterListType parsValue,
                             TciValueTemplate parsTmpl,
                             TciValueDifferenceList diffs,
                             TriComponentId from,
                             TciNonValueTemplate fromTmpl) {}

/* tliPrGetCall_m
 * 
 * Shall be called by TE to log getting a call. This event occurs after getcall has matched against a
 * template. This event is used for logging the communication with the SUT.
*/
                                    
void tliPrGetCall_m (String am,
                     long int ts,
                     String src,
                     long int line,
                     TriComponentId c,
                     TriPortId port,
                     TriSignatureId signature,
                     TciParameterListType parsValue,
                     TciValueTemplate parsTmpl,
                     TriAddress address,
                     TciValueTemplate addressTmpl) {}

/* tliPrGetCall_c
 * 
 * Shall be called by TE to log getting a call. This event occurs after getcall has matched against a
 * template. This event is used for logging the intercomponent communication.
*/
                            
void tliPrGetCall_c (String am,
                     long int ts,
                     String src,
                     long int line,
                     TriComponentId c,
                     TriPortId port,
                     TriSignatureId signature,
                     TciParameterListType parsValue,
                     TciValueTemplate parsTmpl,
                     TriComponentId from,
                     TciNonValueTemplate fromTmpl) {}

/* tliPrReply_m
 * 
 * Shall be called by SA to log a unicast reply operation. This event occurs after reply execution. This
 * event is used for logging the communication with the SUT.
*/

void tliPrReply_m (String am,
                   long int ts,
                   String src,
                   long int line,
                   TriComponentId c,
                   TriPortId atPort,
                   TriPortId toPort,
                   TriSignatureId signature,
                   TciParameterListType parsValue,
                   TciValue replValue,
                   TriAddress address,
                   TriStatus encoderFailure,
                   TriParameterList pars,
                   TriParameter repl,
                   TriStatus transmissionFailure) {}

/* tliPrReply_m_BC
 * 
 * Shall be called by SA to log a broadcast reply operation. This event occurs after reply execution. This
 * event is used for logging the communication with the SUT.
*/
                          
void tliPrReply_m_BC (String am,
                      long int ts,
                      String src,
                      long int line,
                      TriComponentId c,
                      TriPortId atPort,
                      TriPortId toPort,
                      TriSignatureId signature,
                      TciParameterListType parsValue,
                      TciValue replValue,
                      TriStatus encoderFailure,
                      TriParameterList pars,
                      TriParameter repl,
                      TriStatus transmissionFailure) {}

/* tliPrReply_m_MC
 * 
 * Shall be called by SA to log a multicast reply operation. This event occurs after reply execution. This
 * event is used for logging the communication with the SUT.
*/
                             
void tliPrReply_m_MC (String am,
                      long int ts,
                      String src,
                      long int line,
                      TriComponentId c,
                      TriPortId atPort,
                      TriPortId toPort,
                      TriSignatureId signature,
                      TciParameterListType parsValue,
                      TciValue replValue,
                      TriAddressList addresses,
                      TriStatus encoderFailure,
                      TriParameterList pars,
                      TriParameter repl,
                      TriStatus transmissionFailure) {}

/* tliPrReply_c
 * 
 * Shall be called by CH to log a unicast reply operation. This event occurs after reply execution. This
 * event is used for logging the intercomponent communication.
*/
 
void tliPrReply_c (String am,
                   long int ts,
                   String src,
                   long int line,
                   TriComponentId c,
                   TriPortId atPort,
                   TriPortId toPort,
                   TriSignatureId signature,
                   TciParameterListType parsValue,
                   TciValue replValue,
                   TriStatus transmissionFailure) {}

/* tliPrReply_c_BC
 * 
 * Shall be called by CH to log a broadcast reply operation. This event occurs after reply execution. This
 * event is used for logging the intercomponent communication.
*/
                          
void tliPrReply_c_BC (String am,
                      long int ts,
                      String src,
                      long int line,
                      TriComponentId c,
                      TriPortId atPort,
                      TriPortIdList toPortList,
                      TriSignatureId signature,
                      TciParameterListType parsValue,
                      TciValue replValue,
                      TriStatus transmissionFailure) {}

/* tliPrReply_c_MC
 * 
 * Shall be called by CH to log a multicast reply operation. This event occurs after reply execution. This
 * event is used for logging the intercomponent communication.
*/
                             
void tliPrReply_c_MC (String am,
                      long int ts,
                      String src,
                      long int line,
                      TriComponentId c,
                      TriPortId atPort,
                      TriPortIdList toPortList,
                      TriSignatureId signature,
                      TciParameterListType parsValue,
                      TciValue replValue,
                      TriStatus transmissionFailure) {}

/* tliPrGetReplyDetected_m
 * 
 * Shall be called by SA to log the getreply enqueue operation. This event occurs after getreply is
 * enqueued. This event is used for logging the communication with the SUT.
*/
                             
void tliPrGetReplyDetected_m (String am,
                              long int ts,
                              String src,
                              long int line,
                              TriComponentId c,
                              TriPortId atPort,
                              TriPortId fromPort,
                              TriSignatureId signature,
                              TriParameterList pars,
                              TriParameter repl,
                              TriAddress address) {}

/* tliPrGetReplyDetected_c
 * 
 * Shall be called by CH to log the getreply enqueue operation. This event occurs after getreply is
 * enqueued. This event is used for logging the intercomponent communication.
*/
                                     
void tliPrGetReplyDetected_c (String am,
                              long int ts,
                              String src,
                              long int line,
                              TriComponentId c,
                              TriPortId atPort,
                              TriPortId fromPort,
                              TriSignatureId signature,
                              TciParameterListType parsValue,
                              TciValue replValue) {}

/* tliPrGetReplyMismatch_m
 * 
 * Shall be called by TE to log the mismatch of a getreply operation. This event occurs after getreply is
 * checked against a template. This event is used for logging the communication with SUT.
*/
                                     
void tliPrGetReplyMismatch_m (String am,
                              long int ts,
                              String src,
                              long int line,
                              TriComponentId c,
                              TriPortId port,
                              TriSignatureId signature,
                              TciParameterListType parsValue,
                              TciValueTemplate parsTmpl,
                              TciValue replValue,
                              TciValueTemplate replyTmpl,
                              TciValueDifferenceList diffs,
                              TriAddress address,
                              TciValueTemplate addressTmpl) {}

/* tliPrGetReplyMismatch_c
 * 
 * Shall be called by TE to log the mismatch of a getreply operation. This event occurs after getreply is
 * checked against a template. This event is used for logging the intercomponent communication.
*/

void tliPrGetReplyMismatch_c (String am,
                              long int ts,
                              String src,
                              long int line,
                              TriComponentId c,
                              TriPortId port,
                              TriSignatureId signature,
                              TciParameterListType parsValue,
                              TciValueTemplate parsTmpl,
                              TciValue replValue,
                              TciValueTemplate replyTmpl,
                              TciValueDifferenceList diffs,
                              TriComponentId from,
                              TciNonValueTemplate fromTmpl) {}

/* tliPrGetReply_m
 * 
 * Shall be called by TE to log getting a reply. This event occurs after getreply is checked against a
 * template. This event is used for logging the communication with SUT.
*/
                                     
void tliPrGetReply_m (String am,
                      long int ts,
                      String src,
                      long int line,
                      TriComponentId c,
                      TriPortId port,
                      TriSignatureId signature,
                      TciParameterListType parsValue,
                      TciValueTemplate parsTmpl,
                      TciValue replValue,
                      TciValueTemplate replyTmpl,
                      TriAddress address,
                      TciValueTemplate addressTmpl) {}

/* tliPrGetReply_c
 * 
 * Shall be called by TE to log getting a reply. This event occurs after getreply is checked against a
 * template. This event is used for logging the intercomponent communication.
*/
                             
void tliPrGetReply_c (String am,
                      long int ts,
                      String src,
                      long int line,
                      TriComponentId c,
                      TriPortId port,
                      TriSignatureId signature,
                      TciParameterListType parsValue,
                      TciValueTemplate parsTmpl,
                      TciValue replValue,
                      TciValueTemplate replyTmpl,
                      TriComponentId from,
                      TciNonValueTemplate fromTmpl) {}

/* tliPrRaise_m
 * 
 * Shall be called by SA to log a unicast raise operation. This event occurs after reply execution. This
 * event is used for logging the communication with the SUT.
*/
                             
void tliPrRaise_m (String am,
                   long int ts,
                   String src,
                   long int line,
                   TriComponentId c,
                   TriPortId atPort,
                   TriPortId toPort,
                   TriSignatureId signature,
                   TciParameterListType parsValue,
                   TciValue excValue,
                   TriAddress address,
                   TriStatus encoderFailure,
                   TriException exc,
                   TriStatus transmissionFailure) {}

/* tliPrRaise_m_BC
 * 
 * Shall be called by SA to log a broadcast raise operation. This event occurs after reply execution. This
 * event is used for logging the communication with the SUT.
*/
                          
void tliPrRaise_m_BC (String am,
                      long int ts,
                      String src,
                      long int line,
                      TriComponentId c,
                      TriPortId atPort,
                      TriPortId toPort,
                      TriSignatureId signature,
                      TciParameterListType parsValue,
                      TciValue excValue,
                      TriStatus encoderFailure,
                      TriException exc,
                      TriStatus transmissionFailure) {}

/* tliPrRaise_m_MC
 * 
 * Shall be called by SA to log a multicast raise operation. This event occurs after reply execution. This
 * event is used for logging the communication with the SUT.
*/
                             
void tliPrRaise_m_MC (String am,
                      long int ts,
                      String src,
                      long int line,
                      TriComponentId c,
                      TriPortId atPort,
                      TriPortId toPort,
                      TriSignatureId signature,
                      TciParameterListType parsValue,
                      TciValue excValue,
                      TriAddressList addresses,
                      TriStatus encoderFailure,
                      TriException exc,
                      TriStatus transmissionFailure) {}

/* tliPrRaise_c
 * 
 * Shall be called by CH to log a unicast raise operation. This event occurs after reply execution. This
 * event is used for logging the intercomponent communication.
*/
                             
void tliPrRaise_c (String am,
                   long int ts,
                   String src,
                   long int line,
                   TriComponentId c,
                   TriPortId atPort,
                   TriPortId toPort,
                   TriSignatureId signature,
                   TciParameterListType parsValue,
                   TciValue excValue,
                   TriStatus transmissionFailure) {}

/* tliPrRaise_c_BC
 * 
 * Shall be called by CH to log a broadcast raise operation. This event occurs after reply execution. This
 * event is used for logging the intercomponent communication.
*/
                          
void tliPrRaise_c_BC (String am,
                      long int ts,
                      String src,
                      long int line,
                      TriComponentId c,
                      TriPortId atPort,
                      TriPortIdList toPortList,
                      TriSignatureId signature,
                      TciParameterListType parsValue,
                      TciValue excValue,
                      TriStatus transmissionFailure) {}
                             
/* tliPrRaise_c_MC
 * 
 * Shall be called by CH to log a multicast raise operation. This event occurs after reply execution. This
 * event is used for logging the intercomponent communication.
*/

void tliPrRaise_c_MC (String am,
                      long int ts,
                      String src,
                      long int line,
                      TriComponentId c,
                      TriPortId atPort,
                      TriPortIdList toPortList,
                      TriSignatureId signature,
                      TciParameterListType parsValue,
                      TciValue excValue,
                      TriStatus transmissionFailure) {}

/* tliPrCatchDetected_m
 * 
 * Shall be called by SA to log the catch enqueue operation. This event occurs after catch is
 * enqueued. This event is used for logging the communication with the SUT.
*/
                             
void tliPrCatchDetected_m (String am,
                           long int ts,
                           String src,
                           long int line,
                           TriComponentId c,
                           TriPortId atPort,
                           TriPortId fromPort,
                           TriSignatureId signature,
                           TriException exc,
                           TriAddress address) {}

/* tliPrCatchDetected_c
 * 
 * Shall be called by CH to log the catch enqueue operation. This event occurs after catch is enqueued.
 * This event is used for logging the intercomponent communication.
*/
                                  
void tliPrCatchDetected_c (String am,
                           long int ts,
                           String src,
                           long int line,
                           TriComponentId c,
                           TriPortId atPort,
                           TriPortId fromPort,
                           TriSignatureId signature,
                           TciValue excValue) {}

/* tliPrCatchMismatch_m
 * 
 * Shall be called by TE to log the mismatch of a catch operation. This event occurs after catch is
 * checked against a template. This event is used for logging the communication with SUT.
*/
                                  
void tliPrCatchMismatch_m (String am,
                           long int ts,
                           String src,
                           long int line,
                           TriComponentId c,
                           TriPortId port,
                           TriSignatureId signature,
                           TciValue excValue,
                           TciValueTemplate excTmpl,
                           TciValueDifferenceList diffs,
                           TriAddress address,
                           TciValueTemplate addressTmpl) {}

/* tliPrCatchMismatch_c
 * 
 * Shall be called by TE to log the mismatch of a catch operation. This event occurs after catch is
 * checked against a template. This event is used for logging the intercomponent communication.
*/
                                  
void tliPrCatchMismatch_c (String am,
                           long int ts,
                           String src,
                           long int line,
                           TriComponentId c,
                           TriPortId port,
                           TriSignatureId signature,
                           TciValue excValue,
                           TciValueTemplate excTmpl,
                           TciValueDifferenceList diffs,
                           TriComponentId from,
                           TciNonValueTemplate fromTmpl) {}

/* tliPrCatch_m
 * 
 * Shall be called by SA to log catching an exception. This event occurs after catch is checked against
 * a template. This event is used for logging the communication with SUT.
*/
                                  
void tliPrCatch_m (String am,
                   long int ts,
                   String src,
                   long int line,
                   TriComponentId c,
                   TriPortId port,
                   TriSignatureId signature,
                   TciValue excValue,
                   TciValueTemplate excTmpl,
                   TriAddress address,
                   TciValueTemplate addressTmpl) {}

/* tliPrCatch_c
 * 
 * Shall be called by CH to log catching an exception. This event occurs after catch is checked against
 * a template. This event is used for logging the intercomponent communication.
*/
                          
void tliPrCatch_c (String am,
                   long int ts,
                   String src,
                   long int line,
                   TriComponentId c,
                   TriPortId port,
                   TriSignatureId signature,
                   TciValue excValue,
                   TciValueTemplate excTmpl,
                   TriComponentId from,
                   TciNonValueTemplate fromTmpl) {}

/* tliPrCatchTimeoutDetected
 * 
 * Shall be called by PA to log the detection of a catch timeout. This event occurs after the timeout is
 * enqueued.
*/
                          
void tliPrCatchTimeoutDetected (String am,
                                long int ts,
                                String src,
                                long int line,
                                TriComponentId c,
                                TriPortId port,
                                TriSignatureId signature) {}

/* tliPrCatchTimeout
 * 
 * Shall be called by TE to log catching a timeout. This event occurs after the catch timeout has been
 * performed.
*/
                                       
void tliPrCatchTimeout (String am,
                        long int ts,
                        String src,
                        long int line,
                        TriComponentId c,
                        TriPortId port,
                        TriSignatureId signature) {}

/* tliCCreate
 * 
 * Shall be called by TE to log the create component operation. This event occurs after component
 * creation.
*/
                               
void tliCCreate (String am,
                 long int ts,
                 String src,
                 long int line,
                 TriComponentId c,
                 TriComponentId comp,
                 String name,
                 unsigned char alive) {}

/* tliCStart
 * 
 * Shall be called by TE to log the start component operation. This event occurs after component start.
*/
                        
void tliCStart (String am,
                long int ts,
                String src,
                long int line,
                TriComponentId c,
                TriComponentId comp,
                TciBehaviourIdType name,
                TciParameterListType parsValue) {}

/* tliCRunning
 * 
 * Shall be called by TE to log the running component operation. This event occurs after component
 * running.
*/
                       
void tliCRunning (String am,
                  long int ts,
                  String src,
                  long int line,
                  TriComponentId c,
                  TriComponentId comp,
                  ComponentStatus status) {}

/* tliCAlive
 * 
 * Shall be called by TE to log the alive component operation. This event occurs after component
 * alive.
*/

                         
void tliCAlive (String am,
                long int ts,
                String src,
                long int line,
                TriComponentId c,
                TriComponentId comp,
                ComponentStatus status) {}

/* tliCStop
 * 
 * Shall be called by TE to log the stop component operation. This event occurs after component stop.
*/

                       
void tliCStop (String am,
               long int ts,
               String src,
               long int line,
               TriComponentId c,
               TriComponentId comp) {}

/* tliCKill
 * 
 * Shall be called by TE to log the kill component operation. This event occurs after component kill.
*/

                      
void tliCKill (String am,
               long int ts,
               String src,
               long int line,
               TriComponentId c,
               TriComponentId comp) {}

/* tliCDoneMismatch
 * 
 * Shall be called by TE to log the mismatch of a done component operation. This event occurs after
 * done is checked against a template.
*/
                      
void tliCDoneMismatch (String am,
                       long int ts,
                       String src,
                       long int line,
                       TriComponentId c,
                       TriComponentId comp,
                       TciNonValueTemplate compTmpl) {}

/* tliCDone
 * 
 * Shall be called by TE to log the done component operation. This event occurs after the done
 * operation.
*/
                              
void tliCDone (String am,
               long int ts,
               String src,
               long int line,
               TriComponentId c,
               TriComponentId comp,
               TciNonValueTemplate compTmpl) {}

/* tliCKilledMismatch
 * 
 * Shall be called by TE to log the mismatch of a killed component operation. This event occurs after
 * killed is checked against a template.
*/
                      
void tliCKilledMismatch (String am,
                         long int ts,
                         String src,
                         long int line,
                         TriComponentId c,
                         TriComponentId comp,
                         TciNonValueTemplate compTmpl) {}

/* tliCKilled
 * 
 * Shall be called by TE to log the killed component operation. This event occurs after the killed
 * operation.
*/
                                
void tliCKilled (String am,
                 long int ts,
                 String src,
                 long int line,
                 TriComponentId c,
                 TriComponentId comp,
                 TciNonValueTemplate compTmpl) {}

/* tliCTerminated
 * 
 * Shall be called by TE to log the termination of a component. This event occurs after the termination
 * of the component.
*/
                        
void tliCTerminated (String am,
                     long int ts,
                     String src,
                     long int line,
                     TriComponentId c,
                     TciValue verdict) {}

/* tliPConnect
 * 
 * Shall be called by CH to log the connect operation. This event occurs after the connect operation.
*/
                            
void tliPConnect (String am,
                  long int ts,
                  String src,
                  long int line,
                  TriComponentId c,
                  TriPortId port1,
                  TriPortId port2) {}

/* tliPDisconnect
 * 
 * Shall be called by CH to log the disconnect operation. This event occurs after the disconnect 
 * operation.
*/
                         
void tliPDisconnect (String am,
                     long int ts,
                     String src,
                     long int line,
                     TriComponentId c,
                     TriPortId port1,
                     TriPortId port2) {}

/* tliPMap
 * 
 * Shall be called by SA to log the map operation. This event occurs after the map operation.
*/
                            
void tliPMap (String am,
              long int ts,
              String src,
              long int line,
              TriComponentId c,
              TriPortId port1,
              TriPortId port2) {}

/* tliPUnmap
 * 
 * Shall be called by SA to log the unmap operation. This event occurs after the unmap operation.
*/
                     
void tliPUnmap (String am,
                long int ts,
                String src,
                long int line,
                TriComponentId c,
                TriPortId port1,
                TriPortId port2) {}

/* tliPClear
 * 
 * Shall be called by TE to log the port clear operation. This event occurs after the port clear
 * operation.
*/
                       
void tliPClear (String am,
                long int ts,
                String src,
                long int line,
                TriComponentId c,
                TriPortId port) {}

/* tliPStart
 * 
 * Shall be called by TE to log the port start operation. This event occurs after the port start operation.
*/
                       
void tliPStart (String am,
                long int ts,
                String src,
                long int line,
                TriComponentId c,
                TriPortId port) {}

/* tliPStop
 * 
 * Shall be called by TE to log the port stop operation. This event occurs after the port stop operation.
*/
                       
void tliPStop (String am,
               long int ts,
               String src,
               long int line,
               TriComponentId c,
               TriPortId port) {}

/* tliPHalt
 * 
 * Shall be called by TE to log the port halt operation. This event occurs after the port halt operation.
*/
                      
void tliPHalt (String am,
               long int ts,
               String src,
               long int line,
               TriComponentId c,
               TriPortId port) {}

/* tliEncode
 * 
 * Shall be called by CD to log the encode operation.
*/
                      
void tliEncode (String am,
                long int ts,
                String src,
                long int line,
                TriComponentId c,
                TciValue val,
                TriStatus encoderFailure,
                TriMessage msg,
                String codec) {}

/* tliDecode
 * 
 * Shall be called by CD to log the decode operation.
*/
                       
void tliDecode (String am,
                long int ts,
                String src,
                long int line,
                TriComponentId c,
                TriMessage msg,
                TriStatus decoderFailure,
                TciValue val,
                String codec) {}

/* tliTTimeoutDetected
 * 
 * Shall be called by PA to log the detection of a timeout. This event occurs after timeout is
 * enqueued.
*/
                       
void tliTTimeoutDetected (String am,
                          long int ts,
                          String src,
                          long int line,
                          TriComponentId c,
                          TriTimerId timer) {}

/* tliTTimeoutMismatch
 * 
 * Shall be called by TE to log a timeout mismatch. This event occurs after a timeout match failed.
*/
                                 
void tliTTimeoutMismatch (String am,
                          long int ts,
                          String src,
                          long int line,
                          TriComponentId c,
                          TriTimerId timerId,
                          TciNonValueTemplate timerTmpl) {}

/* tliTTimeout
 * 
 * Shall be called by TE to log a timeout match. This event occurs after a timeout matched.
*/
                                 
void tliTTimeout (String am,
                  long int ts,
                  String src,
                  long int line,
                  TriComponentId c,
                  TriTimerId timerId,
                  TciNonValueTemplate timerTmpl) {}

/* tliTStart
 * 
 * Shall be called by PA to log the start of a timer. This event occurs after the start timer
 * operation.
*/
                         
void tliTStart (String am,
                long int ts,
                String src,
                long int line,
                TriComponentId c,
                TriTimerId timer,
                TriTimerDuration dur) {}

/* tliTStop
 * 
 * Shall be called by PA to log the stop of a timer. This event occurs after the stop timer operation.
*/
                       
void tliTStop (String am,
               long int ts,
               String src,
               long int line,
               TriComponentId c,
               TriTimerId timer,
               TriTimerDuration dur) {}

/* tliTRead
 * 
 * Shall be called by PA to log the reading of a timer. This event occurs after the read timer
 * operation.
*/
                      
void tliTRead (String am,
               long int ts,
               String src,
               long int line,
               TriComponentId c,
               TriTimerId timer,
               TriTimerDuration elapsed) {}

/* tliTRunning
 *
 * Shall be called by PA to log the running timer operation. This event occurs after the running timer
 * operation.
*/
                      
void tliTRunning (String am,
                  long int ts,
                  String src,
                  long int line,
                  TriComponentId c,
                  TriTimerId timer,
                  TimerStatus status) {}

/* tliSEnter
 *
 * Shall be called by TE to log the entering of a scope. This event occurs after the scoped has been
 * entered.
*/
                         
void tliSEnter (String am,
                long int ts,
                String src,
                long int line,
                TriComponentId c,
                QualifiedName name,
                TciParameterListType parsValue,
                String kind) {}

/* tliSLeave
 *
 * Shall be called by TE to log the leaving of a scope. This event occurs after the scoped has been
 * leaved.
*/
                       
void tliSLeave (String am,
                long int ts,
                String src,
                long int line,
                TriComponentId c,
                QualifiedName name,
                TciParameterListType parsValue,
                TciValue returnValue,
                String kind) {}

/* tliVar
 *
 * Shall be called by TE to log the modification of the value of a variable. This event occurs after the
 * values has been changed.
*/
                       
void tliVar (String am,
             long int ts,
             String src,
             long int line,
             TriComponentId c,
             QualifiedName name,
             TciValue varValue) {}

/* tliModulePar
 *
 * Shall be called by TE to log the value of a module parameter. This event occurs after the
 * access to the value of a module parameter.
*/
                    
void tliModulePar (String am,
                   long int ts,
                   String src,
                   long int line,
                   TriComponentId c,
                   QualifiedName name,
                   TciValue parValue) {}

/* tliGetVerdict
 *
 * Shall be called by TE to log the getverdict operation. This event occurs after the getverdict
 * operation.
*/
                          
void tliGetVerdict (String am,
                    long int ts,
                    String src,
                    long int line,
                    TriComponentId c,
                    TciValue verdict) {}

/* tliSetVerdict
 *
 * Shall be called by TE to log the setverdict operation. This event occurs after the setverdict
 * operation.
*/
                           
void tliSetVerdict (String am,
                    long int ts,
                    String src,
                    long int line,
                    TriComponentId c,
                    TciValue verdict) {}

/* tliLog
 *
 * Shall be called by TM to log the TTCN-3 statement log. This event occurs after the TTCN-3 log
 * operation.
*/

void tliLog (String am,
             long int ts,
             String src,
             long int line,
             TriComponentId c,
             String log) {}

/* tliAEnter
 *
 * Shall be called by TE to log entering an alt. This event occurs after an alt has been entered.
*/
                    
void tliAEnter (String am,
                long int ts,
                String src,
                long int line,
                TriComponentId c) {}

/* tliALeave
 *
 * Shall be called by TE to log leaving an alt. This event occurs after the alt has been leaved.
*/
                      
void tliALeave (String am,
                long int ts,
                String src,
                long int line,
                TriComponentId c) {}

/* tliADefaults
 *
 * Shall be called by TE to log entering the default section. This event occurs after the default
 * section has been entered.
*/
                       
void tliADefaults (String am,
                   long int ts,
                   String src,
                   long int line,
                   TriComponentId c) {}

/* tliAActivate
 *
 * Shall be called by TE to log the activation of a default. This event occurs after the default activation.
*/
                          
void tliAActivate (String am,
                   long int ts,
                   String src,
                   long int line,
                   TriComponentId c,
                   QualifiedName name,
                   TciParameterListType pars,
                   TciValue ref) {}

/* tliADeactivate
 *
 * Shall be called by TE to log the deactivation of a default. This event occurs after the default
 * deactivation.
*/
                          
void tliADeactivate (String am,
                     long int ts,
                     String src,
                     long int line,
                     TriComponentId c,
                     TciValue ref) {}

/* tliANomatch
 *
 * Shall be called by TE to log the nomatch of an alt. This event occurs after the alt has not matched.
*/
                            
void tliANomatch (String am,
                  long int ts,
                  String src,
                  long int line,
                  TriComponentId c) {}
                         
/* tliARepeat
 *
 * Shall be called by TE to log repeating an alt. This event occurs when the alt is been repeated.
*/
                         
void tliARepeat (String am,
                 long int ts,
                 String src,
                 long int line,
                 TriComponentId c) {}

/* tliAWait
 *
 * Shall be called by TE to log that the component awaits events for a new snapshot.
*/
                        
void tliAWait (String am,
               long int ts,
               String src,
               long int line,
               TriComponentId c) {}

/* tliAction
 *
 * Shall be called by TE to log that the component executed an SUT action.
*/

void tliAction (String am,
                long int ts,
                String src,
                long int line,
                TriComponentId c,
                String action) {}

/* tliMatch
 *
 * Shall be called by TE to log that the component successfully executed a match operation.
*/
                        
void tliMatch(String am,
              long int ts,
              String src,
              long int line,
              TriComponentId c,
              TciValue expr,
              TciValueTemplate tmpl) {}

/* tliMatchMismatch
 *
 * Shall be called by TE to log that the component unsuccessfully executed a match operation - 
 * a mismatch occurred.
*/
                        
void tliMatchMismatch(String am,
                      long int ts,
                      String src,
                      long int line,
                      TriComponentId c,
                      TciValue expr,
                      TciValueTemplate tmpl,
                      TciValueDifferenceList diffs) {}

/* tliInfo
 *
 * Can be called by TE to log additional information during test execution. 
 * This generation of this event is tool dependent as well as the usage of the parameters level and info.
*/
                        
void tliInfo(String am,
             long int ts,
             String src,
             long int line,
             TriComponentId c,
             long int level,
             String info) {}

/* End of File */
