/*************************************************************************
 * Tutorial communication (tut_comm.c)
 *
 * 2010 IBM Corporation
 *
 *************************************************************************/

#ifndef __tut_comm_h
#define __tut_comm_h

#include <t3tri.h>

#ifdef _WIN32
#include <winsock2.h>
#else
typedef int SOCKET;
#define INVALID_SOCKET -1
#define SOCKET_ERROR -1
#endif

#include "tut_ports.h"


#define TUT_SEND_MESSAGE      0
#define TUT_CALL_MESSAGE      1
#define TUT_REPLY_MESSAGE     2


/***********************************************************
 *
 * Socket reader thread data
 *
 ***********************************************************/

typedef struct reader_struct_t {
  SOCKET      socket;
  port_list_t portlist;
} *reader_struct_t;


/***********************************************************
 *
 * Communication functions.
 *
 ***********************************************************/

void
tut_send_message(SOCKET socket,
                 const TriComponentId *componentId, 
                 const TriPortId      *tsiPortId,
                 const TriAddress     *sutAddress, 
                 const TriMessage     *sendMessage);


int
tut_read_message(SOCKET          socket,
                 char **         tsi_port_name, 
                 BinaryString**  data);


void
tut_send_call(SOCKET socket,
              const TriComponentId   *componentId, 
              const TriPortId        *tsiPortId, 
              const TriAddress       *sutAddress, 
              const TriSignatureId   *signatureId,  
              const TriParameterList *parameterList);


void
tut_send_reply(SOCKET socket,
              const TriComponentId   *componentId, 
              const TriPortId        *tsiPortId, 
              const TriAddress       *sutAddress,                 
              const TriSignatureId   *signatureId, 
              const TriParameterList *parameterList, 
              const TriParameter     *returnValue);


#ifdef _WIN32
unsigned tut_socket_reader(void *reader_struct);
#else
void * tut_socket_reader(void *reader_struct);
#endif

#ifndef _WIN32
void Sleep(unsigned long milliseconds);
#endif

#endif
