/*************************************************************************
 * Tutorial port representation and handling (tut_ports.h)
 *
 * 2010 IBM Corporation
 *
 * This module declares the functions and data types used for handling 
 * port mappings.
 *
 * See the implementation of the functions for further documentation.
 *
 *************************************************************************/

#ifndef __tut_ports_h
#define __tut_ports_h

#ifdef _WIN32
#include <windows.h>
#else
#include <pthread.h>
#endif

#include "t3tri.h"

#ifdef DEBUG
#define TRI_PREFIX "INTEGRATION: "
#else
#define TRI_PREFIX ""
#endif

#define TUT_FAILED 1
#define TUT_OK     0


#ifndef assert
#define assert(cond) if (!(cond)) exit(100)
#endif

/***********************************************************
 *
 * Port mapping data types
 *
 ***********************************************************/

typedef struct port_mapping_struct_t 
{
  TriPortId *comp_port;
  TriPortId *tsi_port;
  struct port_mapping_struct_t *next, *prev;
} port_mapping_struct_t, *port_mapping_t;

typedef struct port_list_struct_t
{
  port_mapping_t  list;
#ifdef _WIN32
  HANDLE          mutex;
#else
  pthread_mutex_t mutex;
#endif
} *port_list_t;


/***********************************************************
 *
 * Port mapping macors and functions.
 *
 ***********************************************************/

#define set_port_mapping_comp_port(p,cp) ((p)->comp_port = (cp)) 
#define set_port_mapping_tsi_port(p,i)   ((p)->tsi_port = (i)) 


port_mapping_t
tut_find_port_mapping_noprot(port_list_t ports,
                             const TriPortId *compPortId,
                             const char *tsi_port_name);

port_mapping_t
tut_find_port_mapping(port_list_t ports,
                      const TriPortId *compPortId,
                      const char *tsi_port_name);

void
tut_add_port_mapping(port_mapping_t new_port, port_list_t ports);


void
tut_delete_port_mapping(port_list_t ports, 
                        const TriPortId *compPortId,
                        const TriPortId *tsiPortId);

void
tut_clear_port_mappings(port_list_t ports);


TriPortId *
tut_copy_TriPortId(const TriPortId *portid);

int 
tut_copy_binary(BinaryString *dest, const BinaryString *src);

int 
tut_compare_binary(const BinaryString *bs1, const BinaryString *bs2);



#endif
