
/*
//================================================================
//
//     $Id: asnmisc.typ,v 2.1 2002/07/12 08:13:44 xmso Exp $
//
//     NAME
//          asnmisc.typ
//
//     DESCRIPTION
//          Internal type definitions for the ASN2C Run Time System.
//
//     AUTHORS
//          Pekka Ollikainen (Version 1.nn)
//          Sami Kesti       (Version 1.nn)
//          Markku Turunen
//
//     Copyright (c) Nokia Research Center
//
//     CONTENTS
//          ASNCODG, ASNTSTG and ASNUTIG. For documentation
//          about usage see the ASN2C User's Manual section
//          "Initialization of the RTS".
//
//     NAMING CONVENTIONS
//          If a function is derived from the CASN 1 RTS then
//          name is not changed. Example: COD_INFO, ASNCODG.
//
//          If there is not counterpart type definition in the
//          CASN 1 RTS then a definition has a prefix "ASN".
//          Example: ASNCOD_ERROR_FUNC.
//
//================================================================
*/

#ifndef ASNMISC_TYP
#define ASNMISC_TYP

  /*============================================================*/

  /*
   * INTERNAL DEFINITIONS
   */

#ifndef ASNDEF_DEF
#include "asndef.def"
#endif

#ifndef ASNTYP_TYP
#include "asntyp.typ"
#endif

  /*============================================================*/

#if ASN_RCS_ID
static char asnmisc_typ_rcsid [] =
  "$Id: asnmisc.typ,v 2.1 2002/07/12 08:13:44 xmso Exp $";
#endif

  /*============================================================*/

#if ASN_MEMORY

  /* Memory management handle stack. */
typedef struct asnutig {
  sint16   level;                     /* Number of handles in the stack */
  ASNHOME *homestack [MAX_ASNH_LEV];  /* ASNHOME handle buffer          */
} ASNUTIG;

#endif /* ASN_MEMORY */

  /*============================================================*/

#if ASNTST_MODULE != 0

  /*
   * Even if the rest of the ASN2C RTS would not use sint64 asntst.c 
   * does use it internally.
   */
#ifndef SINT64_H
#include "sint64.h"
#endif

  /* String stack element. */
typedef struct str_stack {
  int              len;         /* Length of a string segment.    */
  BYTE_TYPE        st [TRLEN];  /* String segment value.          */
  struct str_stack *next;
} STR_STACK;

  /* Named number/bit stack element. */
typedef struct named_nbr {
  const char *name;             /* Identifier for a named number. */
  sint32      nbr;              /* Value for a named number.      */
} NAMED_NBR;

  /* Test stack element */
typedef struct tst_info {
  sint32 elemCount;        /* The sequence number of the current  */
                           /* component, element or alternative.  */
  sint32 upperBound;       /* The maximum number of elements.     */
  int    type;             /* One of the TYP_ constants.          */
  int    status;           /* One of the TCS_ constants.          */
} TST_INFO;

  /* Test stack segment. */
typedef struct tst_stack {
  TST_INFO info [MAX_ASN_LEV];  /* Stack elements.                */
  struct tst_stack *prev;       /* Other stack segments.          */
  struct tst_stack *next;
} TST_STACK ;

  /* Input string function for the test module. */
typedef char * (*ASNTST_INPUT_FUNC)  (char *, int, void *);

  /* Output string function for the test module. */
typedef void   (*ASNTST_OUTPUT_FUNC) (const char *, void *);

  /* Test context structure. */
typedef struct asntstg {
  sint32    temp;               /* A temporary variable that is      */
                                /* used in the asntst.ext macros.    */

                                /* Buffers for user answers in the   */
                                /* interactive mode.                 */
  BOOLE     exitAsk;            /* Give XYZ ? (N/y/x) - x            */
  int       noMulti;            /* Give XYZ ? (N/y/x) - n10          */
  int       yesMulti;           /* Give XYZ ? (N/y/x) - y10          */

  int       nrOfPeeked;         /* Number of peeked tokens in the    */
                                /* non-interactive mode.             */
  struct asntstg_token {
    int        code;            /* Token code, TOK_ see asntok.ext   */
    sint32     stringTokenLen;  /* Length of a string token          */
    STR_STACK  stringToken;     /* A string token                    */
    STR_STACK *stringSeg;       /* Current segment in a string token */
    sint32     numberToken;
    sint64     numberToken64;
  } peeked [3];                 /* Storage for three peeked tokens   */

  char     *currPos;            /* Current position in the input     */
                                /* buffer.                           */

  char     *ioStri;             /* String input/output buffer, see   */
                                /* tstStartInStri & tstStartOutStri  */
  sint32    ioLen;              /* Length of the string I/O buffer   */
                                /* still free.                       */

  int       nrErrors;           /* # of errors in non-interactive    */
                                /* mode                              */
  BOOLE     nonInteractive;     /* Is mode interactive or not. See   */
                                /* tstStartOpt and the flag          */
                                /* ASN_TST_NON_INTERACTIVE.          */
  BOOLE     verbose;            /* Is the non-interactive mode       */
                                /* verbose when errors occur. See    */
                                /* the flag ASN_TST_VERBOSE          */
  BOOLE     oldValueNotation;   /* Old choice and any type value     */
                                /* notation in non-interactive mode. */
                                /* The flag ASN_OLD_VALUE_NOTATION.  */
  BOOLE     prettyPrint;        /* Is there extra spaces in bstring  */
                                /* and hstring values so that byte   */
                                /* boundaries are clear. See the     */
                                /* flag ASN_TST_PRETTY_PRINT.        */
  BOOLE     useNewlines;        /* Do we like to have whole output   */
                                /* on one or on multiple lines. See  */
                                /* the flags ASN_TST_USE_NEWLINES    */
                                /* and ASN_TST_NO_NEWLINES.          */
  int       lineNr;             /* Line number, for further use.     */

  sint32    maxNamedNumbers;    /* The largest named number/bit      */
  sint16    nrNamedNumbers;     /* The number of named numbers/bits  */
  NAMED_NBR *namedNumbers;      /* An array of values and ids for    */
                                /* named numbers                     */

  char      inputLine  [TRLEN]; /* Input buffer for file I/O         */
  char      outputLine [TRLEN]; /* Output buffer for file I/O        */
  char      statTypLine [255];  /* Component status information      */
                                /* buffer for interactive mode       */
  char      idLine [255];       /* Component identifier buffer for   */
                                /* interactive mode                  */

                                /* Internal I/O alternatives. For    */
                                /* other ASN.1 tools ilke BEX.       */
  ASNTST_INPUT_FUNC  inFunc;    /* A special input function. See     */
                                /* tstStartInFunc                    */
  void              *inBuffer;  /* A special input buffer. See       */
                                /* tstStartInFunc                    */
  ASNTST_OUTPUT_FUNC outFunc;   /* A special output function. See    */
                                /* tstStartOutFunc                   */
  void              *outBuffer; /* A special output buffer. See      */
                                /* tstStartOutFunc                   */

  int                level;     /* Structure level within ask/print  */
  TST_STACK         *stack;     /* Current test stack segment        */
  TST_INFO          *info;      /* Current test stack element        */
  TST_STACK          basestack; /* The first test stack segment      */

#if ASN_MEMORY
  ASNUTIG           *asnuti;    /* Memory management stack           */
#endif

#if ASNCODEC_MODULE == 1 && ASNPER_MODULE == 1
  BOOLE              isPER;     /* BER or PER                        */
#endif
} ASNTSTG;

#endif /* ASNTST_MODULE != 0 */

  /*============================================================*/

#if ASNCODEC_MODULE != 0

  /* BER encoding/decoding stack element. */
typedef struct cod_info {
  AsnTag    tag;           /* Tag from a ILC triplet.             */
  AsnLength len;           /* Length from a ILC triplet.          */
  AsnLength bytesDecoded;  /* Bytes decoded for this ILC triplet. */
} COD_INFO;

  /* BER encoding/decoding stack segment. */
typedef struct cod_stack {
  COD_INFO info [MAX_ASN_LEV]; /* Stack elements.                 */
#if ASN_RESTR_LEV == 0
  struct cod_stack *prev;      /* Other stack segments            */
  struct cod_stack *next;
#endif
} COD_STACK;


#if ASN_COD_NO_LONGJMP
  /* 
   * Special decoding error reporting function for other ASN.1 
   * tools like BEX.
   */
typedef void (*ASNCOD_ERROR_FUNC) (int         errorCode,
                                   AsnLength   bytesDecoded,
                                   const char *path,
                                   void       *errbuf);
#else
typedef void *ASNCOD_ERROR_FUNC;
#endif

#if ASN_IS_COD_DYNSTR_FUNC
  /* 
   * Special dynamic string allocation function for other ASN.1 
   * tools like BEX.
   */
typedef BYTE_TYPE * (*ASNCOD_DYNSTR_FUNC) (BYTE_TYPE  **val,
                                           AsnMainType  kind,
                                           size_t       len);
#else
typedef BYTE_TYPE *ASNCOD_DYNSTR_FUNC;
#endif

  /* BER encoding/decoding context structure. */
typedef struct asncodg {
  AsnTag       tag;               /* Previously decoded tag.           */
  sint32       temp;              /* A temporary variable that is used */
                                  /* in the d_sint8 macros.            */
  BYTE_TYPE   *bPtr;              /* Pointer to a reserved byte array  */
                                  /* from the current frame.           */
  FRAME_IX     bCnt;              /* Number of bytes used fron the     */
                                  /* reserved array.                   */
  FRAME_PTR   *frame;             /* Current frame buffer.             */
  ERR_RET     *jmpBuf;            /* Current error recovery buffer.    */

#if ASN_REENTRANT == 1 && ASN_MEMORY == 1
  ASNUTIG     *asnuti;            /* Memory management stack.          */
#endif

  sint16       level;             /* Structure level within            */
                                  /* encoding/decoding.                */
  AsnLength    allBytesDecoded;   /* The total # of decoded bytes.     */
  COD_STACK   *stack;             /* Current coding stack segment.     */
  COD_INFO    *info;              /* Current coding stack element.     */
  COD_STACK    basestack;         /* The first coding stack segment.   */

#if ASN_PUSHPATH
  sint16       push;              /* Current length of decoding path.  */
  char         path [MAX_PATH_LEN];
#endif

#if STAND_ALONE_ASN == 0
  BOOLE        doNotFreeFailedAsn; /* CVOPS: in case of decoding error */
                                   /* this flag indicates whether the  */
                                   /* partially decoded value should   */
                                   /* be stored into ASN.1 variables   */
                                   /* or released.                     */
#endif

#if ASN_COD_NO_LONGJMP
  BOOLE             errorHasOccured; /* Error recovery function.      */
  ASNCOD_ERROR_FUNC errorFunc;
  void             *errorBuf;
#endif

#if ASN_IS_COD_DYNSTR_FUNC
  ASNCOD_DYNSTR_FUNC dynStrFunc; /* Dynamic string allocator function. */
#endif
} ASNCODG;

#endif /* ASNCODEC_MODULE */

  /*======================================================*/

#endif /* ASNMISC_TYP */
