/*************************************************************
Generated for IBM Rational Systems Tester 3.3.0.2.17499 at Tue Apr 27 05:07:51 PM MEST 2010
Licensed Materials - Property of IBM 
 Copyright IBM Corporation 2002,2010.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/

/*
//==============================================================
//
//  $Id: sint64.h,v 2.3 2004/03/01 13:39:02 pon Exp $
//
//  NAME
//       sint64
//
//  DESCRIPTION
//       Portable 64 bits wide integer implementation.
//       The implementation is one of the following:
//
//       long
//            Use this if 'long' is 64 bits wide.
//
//       long long
//            Use this if 'long long' is supported.
//
//       __int64
//            Use this if '__int64' is supported.
//
//       struct { long hi, unsigned lo }
//            Otherwise use this. 
//
//  CONFIGURATION
//       Configuration flags, set at most one of them to 1.
//
//       I64_LONG_IS_64_BITS
//            If this flag is set to 1 then the implementation
//            of 'sint64' is 'long'.
//
//       I64_IS_LONG_LONG
//            If this flag is set to 1 then the implementation
//            of 'sint64' is non-standard 'long long'.
//
//       I64_IS_INT64
//            If this flag is set to 1 then the implementation
//            of 'sint64' is non-standard '__int64'.
//
//       If none of the flags is set, then the implementation for 
//       'sint64' is a structure with two 'long' components.
//
//       I64_IS_STR_CONV
//            If this flags is set to 1 then the functions
//            'i64strtosint64' and 'i64tostr' exist.
//
//  FUNCTIONS
//       sint64 i64ltosint64 (long x)
//            Create a sint64 from a long integer.
//
//       sint64 i64ultosint64 (unsigned long x)
//            Create a sint64 from an unsigned long integer.
//
//       int i64strtosint64 (const char *s, sint64 *x)
//            Create a sint64 from a character string value.
//            The format of 's' shall be: "[-]<digit>[<digit>]*".
//            The first digit can be 0 only if there are no more
//            trailing digits. The minus sign is permitted only
//            if the first digit is not 0. If 's' contains a valid
//            value then the value is stored in '*x', and the function
//            returns non-0. Otherwise '*x' shall contain 'i64max()'
//            and the function returns 0.
//
//       sint64 i64make (long hi, unsigned long lo)
//            Create a sint64 from 'hi' and 'lo'.
//
//       int i64tol (sint64 x, long *y)
//            If 'long' can hold value of 'x' then set '*y' to the value
//            and return 1. Otherwise set '*y' to LONG_MAX and return 0.
//
//       int i64toul (sint64 x, unsigned long *y)
//            If 'long' can hold value of 'x' then set '*y' to the value
//            and return 1. Otherwise set '*y' to ULONG_MAX and return 0.
//
//       char *i64tostr (sint64 x, char *buf, size_t bufsize)
//            Return 'x' as a string value. If 'buf' can not hold a 
//            value of 'x' then return NULL.
//
//       long i64hi (sint64 x)
//            Return the 32 high bits.
//
//       unsigned long i64lo (sint64 x)
//            Return the 32 low bits.
//
//       sint64 i64max ()
//            Return the largest positive sint64 value.
//
//       sint64 i64min ()
//            Return the largest negative sint64 value.
//
//       sint64 i64addl  (sint64, long y)
//       sint64 i64addul (sint64, unsigned long y)
//       sint64 i64add   (sint64, sint64 y)
//            Return 'x' + 'y'.
//
//       sint64 i64subl  (sint64, long y)
//       sint64 i64subul (sint64, unsigned long y)
//       sint64 i64sub   (sint64, sint64 y)
//            Return 'x' - 'y'.
//
//       sint64 i64mull  (sint64, long y)
//       sint64 i64mulul (sint64, unsigned long y)
//       sint64 i64mul   (sint64, sint64 y)
//            Return 'x' * 'y'. 
//            NOTE:Multiplication of values near LONG_MIN may result 
//            incorrect results due to internal overflows.
//
//       long i64divl  (sint64, long y)
//       long i64divul (sint64, unsigned long y)
//            Return 'x' / 'y'. 
//            NOTE: Type of the the return value is long.
//
//       sint64 i64modl  (sint64, long y)
//       sint64 i64modul (sint64, unsigned long y)
//            Return 'x' % 'y'.
//
//       sint64 i64neg (sint64 x)
//            Return -'x'.
//
//       int i64iszero (sint64 x)
//            Return  'x' == 0.
//
//       int i64ismax (sint64 x)
//            Return  'x' == 'i64max ()'.
//
//       int i64ismin (sint64 x)
//            Return  'x' == 'i64min ()'.
//
//       int i64ispos (sint64 x)
//            Return  'x' > 0.
//
//       int i64isneg (sint64 x)
//            Return  'x' < 0.
//
//       int i64isgt (sint64 x, sint64 y)
//       int i64isgtl (sint64 x, long y)
//       int i64isgtul (sint64 x, unsigned long y)
//            Return  'x' > 'y'.
//
//       int i64isge (sint64 x, sint64 y)
//       int i64isgel (sint64 x, long y)
//       int i64isgeul (sint64 x, unsigned long y)
//            Return  'x' >= 'y'.
//
//       int i64iseq (sint64 x, sint64 y)
//       int i64iseql (sint64 x, long y)
//       int i64isequl (sint64 x, unsigned long y)
//            Return  'x' == 'y'.
//
//       int i64isle (sint64 x, sint64 y)
//       int i64islel (sint64 x, long y)
//       int i64isleul (sint64 x, unsigned long y)
//            Return  'x' <= 'y'.
//
//       int i64islt (sint64 x, sint64 y)
//       int i64isltl (sint64 x, long y)
//       int i64isltul (sint64 x, unsigned long y)
//            Return  'x' < 'y'.
//
//       int i64isne (sint64 x, sint64 y)
//       int i64isnel (sint64 x, long y)
//       int i64isneul (sint64 x, unsigned long y)
//            Return  'x' != 'y'.
//
//       int i64cmp (sint64 x, sint64 y)
//       int i64cmpl (sint64 x, long y)
//       int i64cmpul (sint64 x, unsigned long y)
//            Return <0, if 'x' < 'y', 0 if 'x' == 'y', >0 if 'x' > 'y'.
//
//       sint64 i64shiftl (sint64 x, unsigned nbits)
//            Shift 'x' to left 'nbits'.
//
//       sint64 i64shiftr (sint64 x, unsigned nbits)
//            Shift 'x' to right 'nbits' as signed integer.
//
//       sint64 i64ushiftr (sint64 x, unsigned nbits)
//            Shift 'x' to right 'nbits' as unsigned signed integer.
//
//       Copyright (C) Nokia Research Center 2001
//
//==============================================================
*/

#ifndef SINT64_H
#define SINT64_H

/*
//==============================================================
//  Configuration flags.
//==============================================================
*/
/*
#ifndef I64_LONG_IS_64_BITS
#define I64_LONG_IS_64_BITS 0
#endif

#ifndef I64_IS_LONG_LONG
#define I64_IS_LONG_LONG    0
#endif

#ifndef I64_IS_INT64
#define I64_IS_INT64        0
#endif
*/

#if defined(_WIN32) && ! defined(WIN32_GCC)
#ifndef I64_IS_INT64
#define I64_IS_INT64 1
#endif
#else
#ifndef I64_IS_LONG_LONG
#define I64_IS_LONG_LONG 1
#endif
#endif

#if (I64_LONG_IS_64_BITS + I64_IS_LONG_LONG + I64_IS_INT64) > 1
# error Set only one of the I64_ configuration flags.
#endif

#define I64_IS_INTEGRAL_TYPE \
  (I64_LONG_IS_64_BITS || I64_IS_LONG_LONG || I64_IS_INT64)

#ifndef I64_IS_STR_CONV
#define I64_IS_STR_CONV 1
#endif

/*
//==============================================================
//  Type definition.
//==============================================================
*/
#if I64_LONG_IS_64_BITS

typedef long sint64;

#endif /* I64_LONG_IS_64_BITS */


#if I64_IS_LONG_LONG

typedef long long sint64;

#endif /* I64_IS_LONG_LONG */

#if I64_IS_INT64

typedef __int64 sint64;

#endif /* I64_IS_INT64 */

#if !I64_IS_INTEGRAL_TYPE

typedef struct {
  long          hi;
  unsigned long lo;
} sint64;

#endif /* I64_IS_INTEGRAL_TYPE */

/*
//==============================================================
//  Functions and macros.
//==============================================================
*/

#include <stdlib.h>
#include <limits.h>

#if I64_IS_INTEGRAL_TYPE

#define i64ltosint64(x)  ((sint64)(x))
#define i64ultosint64(x) ((sint64)(x))

#if I64_LONG_IS_64_BITS
#define i64max()     LONG_MAX
#define i64min()     LONG_MIN
#else
sint64  i64max     (void);
sint64  i64min     (void);
#endif

#define i64addl(x,y)  ((sint64)((x) + (long)(y)))
#define i64addul(x,y) ((sint64)((x) + (unsigned long)(y)))
#define i64add(x,y)   ((sint64)((x) + (y)))

#define i64subl(x,y)  ((sint64)((x) - (long)(y)))
#define i64subul(x,y) ((sint64)((x) - (unsigned long)(y)))
#define i64sub(x,y)   ((sint64)((x) - (y)))

#define i64mull(x,y)  ((sint64)((x) * (long)(y)))
#define i64mulul(x,y) ((sint64)((x) * (unsigned long)(y)))
#define i64mul(x,y)   ((sint64)((x) * (y)))

#define i64divl(x,y)  ((sint64)((x) / (long)(y)))
#define i64divul(x,y) ((sint64)((x) / (unsigned long)(y)))

#define i64modl(x,y)  ((sint64)((x) % (long)(y)))
#define i64modul(x,y) ((sint64)((x) % (unsigned long)(y)))

#define i64neg(x)     (-(x))

#define i64iszero(x)  ((x) == 0)
#define i64ismax(x)   ((x) == i64max())
#define i64ismin(x)   ((x) == i64min())
#define i64ispos(x)   ((x) > 0)
#define i64isneg(x)   ((x) < 0)
#define i64isgt(x,y)  ((x) > (y))
#define i64isge(x,y)  ((x) >= (y))
#define i64iseq(x,y)  ((x) == (y))
#define i64isle(x,y)  ((x) <= (y))
#define i64islt(x,y)  ((x) < (y))
#define i64isne(x,y)  ((x) != (y))

#define i64shiftl(x, nbits) ((sint64)((x) << (nbits)))
#define i64shiftr(x, nbits) ((sint64)((x) >> (nbits)))
sint64  i64ushiftr (sint64 x, unsigned nbits);

#else /* I64_IS_INTEGRAL_TYPE */

sint64        i64ltosint64   (long x);
sint64        i64ultosint64  (unsigned long x);

sint64        i64max     (void);
sint64        i64min     (void);

sint64        i64addl    (sint64 x, long y);
sint64        i64addul   (sint64 x, unsigned long y);
sint64        i64add     (sint64 x, sint64 y);

sint64        i64subl    (sint64 x, long y);
sint64        i64subul   (sint64 x, unsigned long y);
sint64        i64sub     (sint64 x, sint64 y);

sint64        i64mull    (sint64 x, long y);
sint64        i64mulul   (sint64 x, unsigned long y);
sint64        i64mul     (sint64 x, sint64 y);

sint64        i64divl    (sint64 x, long y);
sint64        i64divul   (sint64 x, unsigned long y);

long          i64modl    (sint64 x, long y);
long          i64modul   (sint64 x, unsigned long y);

sint64        i64neg     (sint64 x);

int           i64iszero  (sint64 x);
int           i64ismax   (sint64 x);
int           i64ismin   (sint64 x);
int           i64ispos   (sint64 x);
int           i64isneg   (sint64 x);

#define       i64isgt(x,y)  (i64cmp ((x), (y)) >  0)
#define       i64isge(x,y)  (i64cmp ((x), (y)) >= 0)
#define       i64iseq(x,y)  (i64cmp ((x), (y)) == 0)
#define       i64isle(x,y)  (i64cmp ((x), (y)) <= 0)
#define       i64islt(x,y)  (i64cmp ((x), (y)) < 0)
#define       i64isne(x,y)  (i64cmp ((x), (y)) != 0)

#define       i64isgtl(x,y)  (i64cmpl ((x), (y)) >  0)
#define       i64isgel(x,y)  (i64cmpl ((x), (y)) >= 0)
#define       i64iseql(x,y)  (i64cmpl ((x), (y)) == 0)
#define       i64islel(x,y)  (i64cmpl ((x), (y)) <= 0)
#define       i64isltl(x,y)  (i64cmpl ((x), (y)) < 0)
#define       i64isnel(x,y)  (i64cmpl ((x), (y)) != 0)

#define       i64isgtul(x,y)  (i64cmpul ((x), (y)) >  0)
#define       i64isgeul(x,y)  (i64cmpul ((x), (y)) >= 0)
#define       i64isequl(x,y)  (i64cmpul ((x), (y)) == 0)
#define       i64isleul(x,y)  (i64cmpul ((x), (y)) <= 0)
#define       i64isltul(x,y)  (i64cmpul ((x), (y)) < 0)
#define       i64isneul(x,y)  (i64cmpul ((x), (y)) != 0)

sint64        i64shiftl  (sint64 x, unsigned nbits);
sint64        i64shiftr  (sint64 x, unsigned nbits);
sint64        i64ushiftr (sint64 x, unsigned nbits);

#endif /* I64_IS_INTEGRAL_TYPE */

sint64        i64make    (long a, unsigned long b);
int           i64tol     (sint64 x, long *y);
int           i64toul    (sint64 x, unsigned long *y);
long          i64hi      (sint64 x);
unsigned long i64lo      (sint64 x);

int           i64cmp     (sint64 x, sint64 y);
int           i64cmpl    (sint64 x, long y);
int           i64cmpul   (sint64 x, unsigned long y);

#if I64_IS_STR_CONV
int           i64strtosint64 (const char *s, sint64 *x);
char         *i64tostr       (sint64 x, char *buf, size_t bufsize);
#endif


#endif /* SINT64_H */
