/*************************************************************
Generated for IBM Rational Systems Tester 3.3.0.2.17499 at Tue Apr 27 05:07:51 PM MEST 2010
Licensed Materials - Property of IBM 
 Copyright IBM Corporation 2002,2010.  All Rights Reserved.

U.S. Government Users Restricted Rights:  Use, duplication or disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*************************************************************/
/*---------------------------------------------------------------------------
 * File Name: t3rts_components.h
 */

#ifndef __t3rts_components_h
#define __t3rts_components_h

#ifdef __cplusplus
extern "C" {
#endif

#include "t3rts_common.h"
 
#include "t3rts_context.h"
#include "t3rts_values_general.h"




/*
 * This is a direct mapping from the TTCN-3 'create' operation.
 *
 * This function will create the thread of execution, the control port
 * of this component and initializes the component value with the
 * control port address.  After this operation the created component
 * will be fully initialized and in a state where it is listening to
 * its control port.  Only used for creating PTCs.
 */

t3rt_value_t
t3rt_component_create(t3rt_value_t component,
                      t3rt_value_t name,
                      t3rt_boolean_t is_alive,
                      t3rt_context_t ctx);


/*
 * This is a direct mapping from the TTCN-3 'execute' operation.
 *
 * The return value is the verdict of the executed test case.
 *
 * The component type of the test case is retreived from the symbol
 * table and a call is made to the 't3rt_component_create' to create
 * the MTC.  A 'start' message is sent (simliar to the one sent in
 * 't3rt_component_start'), optionally starts a timer and then enters
 * a loop waiting (primarily) for the final verdict of the test case.
 */

t3rt_value_t
t3rt_component_execute (const char * module, 
                        const char * function,
                        unsigned long params_count, 
                        t3rt_value_t params [],
                        double timeout, 
                        t3rt_context_t ctx);

/* 
 * Calls function with parameters on specified component.  This is done
 * by sending a 'start' message to that component.  Only used for
 * starting PTCs.
 */

void
t3rt_component_start (t3rt_value_t component,
                      const char * module, 
                      const char * function,
                      unsigned long params_count, 
                      t3rt_value_t params [],
                      t3rt_context_t ctx);


/* 
 * This is the function to be used in the 't3pl_task_create' as the
 * main function.  It needs the 'control_port_address' to be able to
 * create the context of the new component.
 */

void
t3rt_component_main(t3rt_binary_string_t control_port_address,
                    t3rt_context_t ctx);


                    
/*
 * Stops the current component.  In case of MTC, stops all the PTCs as well.
 */

void
t3rt_component_stop(t3rt_value_t comp_ref, t3rt_context_t ctx);

/*
 * Kills the current component.  In case of MTC, stops all the PTCs as well.
 */

void
t3rt_component_kill(t3rt_value_t comp_ref, t3rt_context_t ctx);

/*
 * Checks if the specified component is running.  The MTC can call
 * this function with the 'component' parameter set to the
 * 't3rt_value_any_c' or the 't3rt_value_all_c' constant.
 */

t3rt_boolean_t
t3rt_component_is_running(t3rt_value_t component, t3rt_context_t ctx);

/*
 * Checks if the specified component is alive.  The MTC can call
 * this function with the 'component' parameter set to the
 * 't3rt_value_any_c' or the 't3rt_value_all_c' constant.
 */

t3rt_boolean_t
t3rt_component_is_alive(t3rt_value_t component, t3rt_context_t ctx);

/*
 * Checks if the specified component is done within an alternative
 * statement.  The MTC can call this function with the 'component'
 * parameter set to the 't3rt_value_any_c' or the 't3rt_value_all_c'
 * constant.
 */

t3rt_boolean_t
t3rt_component_try_done(t3rt_value_t component, t3rt_context_t ctx);

/*
 * Checks if the specified component is killed within an alternative
 * statement.  The MTC can call this function with the 'component'
 * parameter set to the 't3rt_value_any_c' or the 't3rt_value_all_c'
 * constant.
 */

t3rt_boolean_t
t3rt_component_try_killed(t3rt_value_t component, t3rt_context_t ctx);

/*
 * Marks an else statement within an alternative statement.
 */

t3rt_boolean_t
t3rt_component_try_else(t3rt_context_t ctx);


/*
 * Returns current component.
 */

t3rt_value_t
t3rt_component_self(t3rt_context_t ctx);


/* 
 * Returns current MTC component.
 */

t3rt_value_t
t3rt_component_mtc(t3rt_context_t ctx);


/* 
 * Returns current system component.
 */

t3rt_value_t
t3rt_component_system(t3rt_context_t ctx);


/*
 * Sets local verdict.
 */

void t3rt_component_set_local_verdict(t3rt_value_t verdict_value, t3rt_value_t values[], t3rt_context_t ctx);


/*
 * Returns local verdict.
 */ 

t3rt_value_t
t3rt_component_get_local_verdict(t3rt_context_t ctx);


/*
 * Returns indicated component element (port record, variable or timer) 
 * value.
 */

t3rt_value_t
t3rt_component_element(const char* element, t3rt_context_t ctx);


/*
 * Takes snapshot for the current component.  This function should
 * only be called before checking for existing events for the available
 * set of alternatives.
 */

void
t3rt_component_snapshot(t3rt_context_t ctx);


/*
 * Blocks the current component waiting for some external stimuli
 * (i.e. some message received or timer timing out).  This function
 * should only be called after checking for existing events for the
 * available set of alternatives.  If the flag busy_wait is set, then
 * this will both disable deadlock detection and also prevent this
 * function from really waiting for additional input.
 */

void
t3rt_component_wait(t3rt_boolean_t busy_wait, t3rt_context_t ctx);

/*
 * Gives the RTS a chance to handle pending control messages. 
 * The intended usage is inside loop constructs and functions (infinite 
 * recursion...) so that they don't wait too long before responding to 
 * control messages.
 */

void
t3rt_component_control(t3rt_context_t ctx);


/*
 * Connects the specified ports of the given components.
 */

void
t3rt_component_connect_port(t3rt_value_t comp1, const char *port1,
                            t3rt_value_t idx1,
                            t3rt_value_t comp2, const char *port2,
                            t3rt_value_t idx2,
                            t3rt_context_t ctx);


/*
 * Maps specified ports.  One of the component values should be the
 * system component.
 */

void
t3rt_component_map_port(t3rt_value_t comp1, const char *port1,
                        t3rt_value_t idx1,
                        t3rt_value_t comp2, const char *port2,
                        t3rt_value_t idx2,
                        t3rt_context_t ctx);


/*
 * Disconnects specified ports.
 */

void
t3rt_component_disconnect_port(t3rt_value_t comp1, const char *port1,
                               t3rt_value_t idx1,
                               t3rt_value_t comp2, const char *port2,
                               t3rt_value_t idx2,
                               t3rt_context_t ctx);


/*
 * Unmaps specified ports.  One of the component values should be the
 * system component.
 */

void
t3rt_component_unmap_port(t3rt_value_t comp1, const char *port1,
                          t3rt_value_t idx1,
                          t3rt_value_t comp2, const char *port2,
                          t3rt_value_t idx2,
                          t3rt_context_t ctx);


/*
 * Set the component type of the system component.
 */

void
t3rt_component_set_system_component_type(t3rt_type_t type, t3rt_context_t ctx);



/*
 * Turns on/off logging of events on the component
 */

void
t3rt_component_mute(t3rt_boolean_t on_off, t3rt_context_t ctx);

#ifdef __cplusplus
}
#endif

#endif

/* End of File */

