/*---------------------------------------------------------------------------
 * File Name: example_components.h
 *
 * 2010 IBM Corporation
 *
 * Description:
 * XXX
 */


#ifndef __example_components_h
#define __example_components_h

#ifdef __cplusplus
extern "C" {
#endif

#include "t3rts.h"

typedef struct ex_task_info_internal_t * ex_task_info_t;
typedef struct ex_port_repr_internal_t * ex_port_repr_t;
typedef struct ex_timer_repr_internal_t * ex_timer_repr_t;

typedef unsigned long ex_component_object_id_t;

#include "example_sequence.h"
#include "example_eventqueue.h"
#include "example_semaphore.h"


#define ex_component_no_object (ex_component_object_id_t)~0

#define ex_component_object_id(COMPIDX, OBJIDX) ((COMPIDX)<<16 | (OBJIDX))
#define ex_component_index(COMPOBJID) (unsigned long)((COMPOBJID) >> 16)
#define ex_object_index(COMPOBJID) (unsigned long)(0xFFFF & (COMPOBJID))


#define EX_CPC_COMPIDX 0
#define EX_MTC_COMPIDX 1
#define EX_SYS_COMPIDX 2


typedef struct ex_port_internal_t
{
  ex_component_object_id_t  id;
  t3rt_binary_string_t      address;
  const char *              port_name;
  long                      port_index; /* -1 for unused */
  t3rt_boolean_t            is_active;
  /* May need rw semaphore to protect against multiple simultaneous map: */
  ex_rw_semaphore_struct_t  sem;
  ex_component_object_id_t  associated_with;
  ex_sequence_head(struct ex_port_internal_t *) sysport_associated_with;
  ex_port_repr_t            port_repr;
} ex_port_struct_t, *ex_port_t;


typedef struct ex_timer_internal_t
{
  ex_component_object_id_t  id;
  t3rt_timer_handle_t       handle;
  ex_timer_repr_t           timer_repr;
} ex_timer_struct_t, *ex_timer_t;


typedef struct ex_component_port_list_internal_t
{
  ex_sequence_head(ex_port_struct_t) ports;
} ex_component_port_list_struct_t, *ex_component_port_list_t;


typedef struct ex_component_timer_list_internal_t
{
  ex_sequence_head(ex_timer_struct_t) timers;
} ex_component_timer_list_struct_t, *ex_component_timer_list_t;


typedef struct ex_component_queue_internal_t
{
  ex_component_object_id_t  id;
  ex_eventqueue_t           events;
} ex_component_queue_struct_t, *ex_component_queue_t;



typedef struct ex_component_internal_t
{
  t3rt_context_t                    ctx;
  ex_component_port_list_struct_t   component_ports;
  ex_component_timer_list_struct_t  component_timers;
  ex_component_queue_t              component_queue;
  ex_task_info_t                    task_info;
} ex_component_struct_t, *ex_component_t;


void
ex_component_pre_initialize(t3rt_context_t ctx);

void
ex_component_initialize(t3rt_context_t ctx);

void
ex_component_finalize(t3rt_context_t ctx);

void
ex_component_release_components(t3rt_context_t ctx);

void
ex_component_release_cpc(t3rt_context_t ctx);

unsigned long
ex_component_create(long number_of_ports, /* -1 for 'unknown' */
                    long number_of_timers, /* -1 for 'unknown' */
                    t3rt_context_t ctx);


void
ex_component_delete_MT_unsafe(unsigned long compidx, t3rt_context_t ctx);

void
ex_component_terminate_MT_unsafe(unsigned long compidx, t3rt_context_t ctx);

void
ex_component_set_context(unsigned long compidx,
                         t3rt_context_t ctx);

t3rt_context_t
ex_component_get_context(unsigned long compidx, t3rt_context_t ctx);

t3rt_context_t
ex_component_get_context_MT_unsafe(unsigned long compidx);

ex_component_port_list_t
ex_component_get_ports(unsigned long compidx, t3rt_context_t ctx);

ex_port_t
ex_component_get_port(ex_component_object_id_t port_object_id,
                      t3rt_context_t ctx);

ex_port_t 
ex_component_get_control_port(unsigned long compidx, t3rt_context_t ctx);

ex_component_timer_list_t
ex_component_get_timers(unsigned long compidx, t3rt_context_t ctx);

ex_timer_t
ex_component_get_timer(ex_component_object_id_t timer_object_id,
                       t3rt_context_t ctx);

ex_component_queue_t
ex_component_get_comp_queue(unsigned long compidx, t3rt_context_t ctx);

ex_task_info_t
ex_component_get_task_info(unsigned long compidx, t3rt_context_t ctx);

void
ex_component_set_task_info(unsigned long compidx, 
                           ex_task_info_t task_info, 
                           t3rt_context_t ctx);

ex_port_t
ex_component_create_port(unsigned long compidx,
                         const char * port_name,
                         long port_index, /* -1 for unused */
                         t3rt_context_t ctx);

void
ex_component_destroy_port(ex_port_t port, t3rt_context_t ctx);

ex_timer_t
ex_component_create_timer(unsigned long compidx, t3rt_context_t ctx);

void
ex_component_destroy_timer(ex_timer_t timer, t3rt_context_t ctx);

void
ex_component_writer_start(t3rt_context_t ctx);

void
ex_component_writer_end(t3rt_context_t ctx);

void
ex_component_reader_start(t3rt_context_t ctx);

void
ex_component_reader_end(t3rt_context_t ctx);

unsigned long
ex_component_get_index(t3rt_binary_string_t address, t3rt_context_t ctx);

ex_component_object_id_t
ex_component_get_port_compobj(t3rt_binary_string_t port_address,
                              t3rt_context_t ctx);

t3rt_binary_string_t
ex_component_get_port_address(ex_component_object_id_t objid,
                              t3rt_context_t ctx);

ex_port_t
ex_component_find_port(unsigned long compidx,
                       const char* name, 
                       long array_index, 
                       t3rt_context_t ctx);

unsigned long
ex_components_find_associated_ports(ex_component_object_id_t pid, 
                                    ex_port_t assoc_ports[], 
                                    unsigned long assoc_ports_length, 
                                    t3rt_context_t ctx);

#ifdef __cplusplus
}
#endif
                                    
#endif

/* End of File */

