/*---------------------------------------------------------------------------
 * File Name: example_eventqueue.h
 *
 * 2010 IBM Corporation
 *
 * Description:
 * This module contains a FIFO event queue, definition of the queue elements 
 * and associated synchronization mechanisms.
 */


#ifndef __example_eventqueue_h
#define __example_eventqueue_h

#ifdef __cplusplus
extern "C" {
#endif


#include "t3rts.h"

typedef struct ex_eventqueue_element_struct_internal_t *
  ex_eventqueue_element_t;
typedef struct ex_eventqueue_struct_internal_t *
  ex_eventqueue_t;



typedef enum ex_eventqueue_element_kind_internal_t
{
  ex_eventqueue_element_message = 1,
  ex_eventqueue_element_call,
  ex_eventqueue_element_reply,
  ex_eventqueue_element_exception,
  ex_eventqueue_element_timeout
} ex_eventqueue_element_kind_t ;


/* 
 *  The queue element containing the component message. 
 *  Contains the union of all the data needed to be communicated to a
 *  component.
 */

typedef struct ex_eventqueue_element_struct_internal_t
{
  ex_eventqueue_element_kind_t kind;
  t3rt_binary_string_t  port_address;
  t3rt_type_t           signature_type;
  t3rt_binary_string_t  data;
  t3rt_binary_string_t *data_vector;
  unsigned long         data_vector_length;
  t3rt_binary_string_t  sender;
  t3rt_timer_handle_t   timer_handle;

  ex_eventqueue_element_t    next;
} ex_eventqueue_element_struct_t;



ex_eventqueue_t
ex_eventqueue_create(t3rt_context_t ctx);

void
ex_eventqueue_clear(ex_eventqueue_t queue, t3rt_context_t ctx);

void
ex_eventqueue_inactivate(ex_eventqueue_t queue, t3rt_context_t ctx);

void
ex_eventqueue_delete(ex_eventqueue_t* queue, t3rt_context_t ctx);

unsigned long
ex_event_size(ex_eventqueue_t queue, t3rt_context_t ctx);



ex_eventqueue_element_t
ex_eventqueue_element_create(ex_eventqueue_element_kind_t kind,
                             t3rt_context_t ctx);

void
ex_eventqueue_element_delete(ex_eventqueue_element_t * elem, 
                             t3rt_context_t ctx);

void 
ex_event_put(ex_eventqueue_t         queue, 
             ex_eventqueue_element_t element,
             t3rt_context_t          ctx);

ex_eventqueue_element_t
ex_event_get(ex_eventqueue_t queue, t3rt_binary_string_t dest_port_address,
             t3rt_context_t  ctx);

ex_eventqueue_element_t 
ex_event_tryget(ex_eventqueue_t queue, 
                t3rt_binary_string_t dest_port_address,
                double * timeout,
                t3rt_context_t  ctx);




void
ex_eventqueue_element_set_port_address(ex_eventqueue_element_t element,
                                       t3rt_binary_string_t    port_addr,
                                       t3rt_context_t          ctx);
void
ex_eventqueue_element_set_signature_type(ex_eventqueue_element_t element,
                                         t3rt_type_t             type,
                                         t3rt_context_t          ctx); 
void
ex_eventqueue_element_set_data(ex_eventqueue_element_t element,
                               t3rt_binary_string_t    data,
                               t3rt_context_t          ctx);    
void
ex_eventqueue_element_set_data_vector(ex_eventqueue_element_t  element,
                                      t3rt_binary_string_t *   data_vector,
                                      unsigned long            size,
                                      t3rt_context_t           ctx);
void
ex_eventqueue_element_set_sender(ex_eventqueue_element_t element,
                                 t3rt_binary_string_t    sender,
                                 t3rt_context_t          ctx);    
void
ex_eventqueue_element_set_timer_handle(ex_eventqueue_element_t element,
                                       t3rt_timer_handle_t     handle,
                                       t3rt_context_t          ctx);

#ifdef __cplusplus
}
#endif
                                       
#endif


/* End of File */

