/*---------------------------------------------------------------------------
 * File Name: t3pl_memory.c
 *
 * 2010 IBM Corporation
 * 
 * Description:
 *     This is an example implementation of the memory part of the
 * Platform Layer.
 */


#include <stdlib.h>


#include "t3rts.h"


/*
 * After a call to this function, the 't3rt_memory_allocate' function must be working.
 */

void
t3pl_memory_pre_initialize (int argc, char * argv [])
{
}


/*
 * After a call to this function, all memory primitives must be available.
 */

void
t3pl_memory_initialize(t3rt_context_t ctx)
{
}


/*
 * Finalizes memory module.
 */

void
t3pl_memory_finalize(t3rt_context_t ctx)
{
}

/*
 * Allocate 'size' number of bytes
 */

void *
t3pl_memory_allocate (const t3rt_alloc_strategy_t strategy,
                      const unsigned long size, 
                      t3rt_context_t context)
{
    void *  mem = NULL;

    switch (strategy) {
    case t3rt_perm_alloc_c:
    case t3rt_static_alloc_c:
        mem = malloc (size);
        break;
    default:
        mem = NULL;
    }

    return mem;
}


/*
 * Deallocate the given memory block
 */

void
t3pl_memory_deallocate (const t3rt_alloc_strategy_t strategy,
                        void *mem, 
                        t3rt_context_t context)
{
    switch (strategy) {
    case t3rt_perm_alloc_c:
    case t3rt_static_alloc_c:
        free (mem);
        break;
    default:
        ;
    }
}


/*
 * Reallocate the given memory block
 */

void *
t3pl_memory_reallocate (const t3rt_alloc_strategy_t strategy,
                        void *mem, 
                        const unsigned long new_size, 
                        t3rt_context_t context)
{
    void *  new_mem = NULL;

    switch (strategy) {
    case t3rt_perm_alloc_c:
    case t3rt_static_alloc_c:
        new_mem = realloc (mem, new_size);
        break;
    default:
        mem = NULL;
    }

    return new_mem;
}

/* End of File */

