/*---------------------------------------------------------------------------
 * File Name: example_semaphore.h
 *
 * 2010 IBM Corporation
 *
 * Description:
 * This is an example implementation of a semaphore utility abstraction.
 *
 * A semaphore is a data object used for managing access to a shared
 * resource between multiple threads.
 *
 * There is a fundamental semaphore functionality available via the
 * t3pl_sem_* functions provided by this module, that this module also
 * uses to build upon to provide so called reader-writer semaphores.
 * This enhanced type of semaphore is used to manage access to a
 * shared resource that can have multiple readers accessing the
 * resource simultaneously, but only a single writer at a time.
 *
 * The readers-writers semaphores are used by first allocating an
 * ex_rw_semaphore_struct structure and initialize it by calling the
 * ex_sem_alloc function and later using the result in other
 * ex_sem_*{start,end} function calls.
 *
 */


#ifndef __example_semaphore_h
#define __example_semaphore_h

#ifdef __cplusplus
extern "C" {
#endif

#include "t3rts.h"


/* =========================================================================
 * Reader-writer semaphores
 * =========================================================================
 */


typedef struct ex_rw_semaphore_struct_internal_t
{
  void * req_sem;
  void * read_sem;
  void * write_sem;
  unsigned long num_readers;
} ex_rw_semaphore_struct_t, *ex_rw_semaphore_t;


void
ex_sem_alloc(ex_rw_semaphore_t rw_sem, t3rt_context_t ctx);

void
ex_sem_dealloc(ex_rw_semaphore_t rw_sem, t3rt_context_t ctx);

void
ex_sem_reader_start(ex_rw_semaphore_t rw_sem, t3rt_context_t ctx);

void
ex_sem_reader_end(ex_rw_semaphore_t rw_sem, t3rt_context_t ctx);

void
ex_sem_writer_start(ex_rw_semaphore_t rw_sem, t3rt_context_t ctx);

void
ex_sem_writer_end(ex_rw_semaphore_t rw_sem, t3rt_context_t ctx);

#ifdef __cplusplus
}
#endif

#endif

/* End of File */

