/*---------------------------------------------------------------------------
 * File Name: example_sequence.c
 *
 * 2010 IBM Corporation
 *
 * Description:
 *  This is an example implementation of a sequence container utility.
 */


#include <string.h>             /* For memcpy, memmove etc */

#include "example_general.h" /* For assert definition */
#include "example_sequence.h"



static void
sequence_ensure_size(unsigned long size, ex_sequence_head_struct_t* head,
                    unsigned long element_size, t3rt_alloc_strategy_t strategy,
                    t3rt_context_t ctx)
{
  if (size > head->alloc) {
    void*               old_content  = head->seq;
    unsigned long       new_capacity = head->alloc * 2;

    if (new_capacity < 5)
      new_capacity = 5;

    assert(size > head->size);
    if (old_content == NULL) {
      head->seq = t3pl_memory_allocate(strategy, element_size * new_capacity,
                                       ctx);
    } else {
      head->seq = t3pl_memory_reallocate(strategy, old_content,
                                         element_size * new_capacity, ctx);
    }

    head->alloc = new_capacity;
  }
}

void
ex_sequence_insert_aux(void *arg, long idx, unsigned long element_size,
                       t3rt_alloc_strategy_t strategy, t3rt_context_t ctx)
{
  ex_sequence_head_struct_t* head = (ex_sequence_head_struct_t*)arg;
  char*                   content = (char*)head->seq;
  unsigned long              uidx = (unsigned long)idx;

  assert(idx >= 0 && idx <= (long)head->size);

  sequence_ensure_size(head->size + 1, head, element_size, strategy, ctx);

  if (uidx < head->size)
    memmove(content + ((uidx + 1) * element_size),
            content + (uidx * element_size),
            ((head->size - uidx) * element_size) );

  head->size += 1;
}


void
ex_sequence_remove_aux(void *arg, long idx, unsigned long element_size,
                       t3rt_context_t ctx)
{
  ex_sequence_head_struct_t* head = (ex_sequence_head_struct_t*)arg;
  char*                   content = (char*)head->seq;
  unsigned long              uidx = (unsigned long)idx;

  assert(idx >= 0 && idx < (long)head->size);

  head->size -= 1;

  if (uidx < head->size)
    memmove(content + (uidx * element_size),
            content + ((uidx + 1) * element_size),
            (head->size - uidx) * element_size);
}


void
ex_sequence_deallocate_aux(void *arg, t3rt_alloc_strategy_t strategy,
                           t3rt_context_t ctx)
{
  ex_sequence_head_struct_t* head = (ex_sequence_head_struct_t*)arg;

  if (strategy == t3rt_temp_alloc_c)
    return;

  if (head->seq != NULL) {
      if (strategy == t3rt_perm_alloc_c) {
        t3pl_memory_deallocate(strategy, head->seq, ctx);
      }
  }

  ex_sequence_init(*head);
}


/* End of File */
