# --------------------------------------------------------------------
# File : ObjectList.tcl
# 
# Desc : This file contains a sample TCL script used to list the
#        TTCN objects in a TTCN project. The script traverses
#        the project and for each object found it displays some
#        information about the object in the log area.
# --------------------------------------------------------------------



# --------------------------------------------------------------------
# Proc : BooleanToString
# Desc : This is a small utility procedure used to convert boolean
#        values to the strings "True" or "False".
# --------------------------------------------------------------------

proc BooleanToString {bool} {
  if { $bool == 1 } {
    return "True"
  } 
  else {
    return "False"
  }
}



# --------------------------------------------------------------------
# Proc : GetPath
# Desc : This procedure is used to get the path of a given TTCN
#        object in order to be able to print this out for the user.
# --------------------------------------------------------------------

proc GetPath {identObj} {
  if { [Class $identObj] == "Module" } {
    return ""
  }

  set composite [GetRole $identObj composite]
  set name ""

  if { [Class $composite] == "Group" } {
    set name [GetPath $composite]/[Get $composite name]
  } elseif { [Class $composite] == "Module" } {
    set name [Get $composite name]
  }

  return $name
}



# --------------------------------------------------------------------
# Proc : ForEachModuleElement
# Desc : This procedure is the one that is called for each module
#        element in order to print out its information. If the module
#        element is a group, that group is traversed recursivelly.
# --------------------------------------------------------------------

proc ForEachModuleElement {moduleElement} {
  
  if { [Class $moduleElement] == "Group" } {
    foreach item $moduleElement {
      MapRole $item moduleElement ForEachModuleElement
    }
  }
  
  if { [Class $moduleElement] != "Comment" } {
    Report $moduleElement [Class $moduleElement] [GetPath $moduleElement] [Get $moduleElement lineNumber] [Get $moduleElement endLine] [Get $moduleElement upToDateStamp]
  }
  
  return 1
}



# --------------------------------------------------------------------
# Proc : ForEachModule
# Desc : This procedure is the one used to traverse all the modules
#        in a TTCN project.
# --------------------------------------------------------------------

proc ForEachModule {module} {
  MapRole $module moduleElement ForEachModuleElement
  return 1
}



# --------------------------------------------------------------------
# Proc : ReportInit
# Desc : Initilization of the format of the report.
# --------------------------------------------------------------------

ReportInit Objects "TTCN Objects" 300 0 "Kind" 150 0 "Path" 300 0 "Start" 40 0 "End" 40 0 "Up to date" 65 0



# --------------------------------------------------------------------
# Proc : Main
# Desc : This is the main loop that traverses the whole project.
# --------------------------------------------------------------------

foreach item $TTCN3 {
  MapRole $item module ForEachModule
}
